/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * test_mqueue.c
 * -------------
 *
 * Test the unicode character handling stuff
 */

#include <stdio.h>

#include "impulse/util/memory.h"
#include "impulse/util/thread.h"
#include "impulse/util/main.h"
#include "impulse/util/hash.h"
#include "impulse/util/error.h"


void print_unicode(impulse_unistring s)
{
  int x;
  
  for (x=0;x<s->length;x++)
    printf("U+%04x ", s->data[x]);
}


impulse_int main(impulse_int argc, 
		 impulse_char *argv[])
{
  impulse_unistring encoding = NULL;
  impulse_unistring  str;
  impulse_char *out;
  impulse_int i;

  impulse_init(argc, argv, "unicode");
  impulse_command_parse();
  
  encoding = impulse_unistring_create_utf("iso885915", 8);
  str = impulse_unistring_create_8bit("xHello world", encoding);
  str->data[0] = 0x20AC; /* Euro sign */
  printf("In Unicode: ");
  print_unicode(str);
  printf("\n");
  out = impulse_unistring_to_8bit(str, encoding);
  
  printf("In Latin-1: %s\n", out);
  impulse_free(out);
  
  out = impulse_unistring_to_utf(str, 8, 0);
  printf("In UTF-8: %s\n", out);
  {
    impulse_unistring foo = impulse_unistring_create_utf(out, 8);
    printf("Back to Unicode: ");
    print_unicode(foo);
    printf("\n");
    impulse_unistring_destroy(foo);
  }
  impulse_free(out);

  out = impulse_unistring_to_utf(str, 7, 0);
  printf("In UTF-7: %s\n", out);
  {
    impulse_unistring foo = impulse_unistring_create_utf(out, 7);
    printf("Back to Unicode: ");
    print_unicode(foo);
    printf("\n");
    impulse_unistring_destroy(foo);
  }
  impulse_free(out);
  
  out = impulse_unistring_to_utf(str, 7, 7);
  printf("In UTF-7 (pure base64): %s\n", out);
  {
    impulse_unistring foo = impulse_unistring_create_utf(out, 7);
    printf("Back to Unicode: ");
    print_unicode(foo);
    printf("\n");
    impulse_unistring_destroy(foo);
  }
  impulse_free(out);
  
  /* Check compositions */
  {
    impulse_unistring str = impulse_unistring_create(
						     &(struct _impulse_unistring){
						       14,
						       14,
						       (impulse_unichar[]) { 0x0041, 0x0301, 0x0042, 0x0045,
							 0x0300, 0x0033, 0x2044, 0x0034,
							 0x0031, 0x2044, 0x0043, 0x0031,
							 0x2044, 0x0033, 0x0000 }});

    printf("Start string: ");
    print_unicode(str);
    printf("\n");
    impulse_unistring_compose(str);
    printf("Composed: ");
    print_unicode(str);
    printf("\n");
    impulse_unistring_decompose(str);
    printf("Decomposed: ");
    print_unicode(str);
    printf("\n");
    impulse_unistring_destroy(str);
  }

  impulse_unistring_destroy(str);

  str = impulse_unistring_create_utf("xTesting\t 1 2 3x\b, End!\n", 7);
  str->data[0] = 0xbd; /* Vulgar fraction 1/2 */;
  str->data[2] = 0xea; /* Small 'e' acute */;
  out = impulse_unistring_to_utf(str, 7, 0);
  printf("Analysing string %s\n", out);
  impulse_free(out);
  
  printf("Pos Letter Ascii Control Alpha Number Space Punct Category Direction Combining Formatting\n");
  for (i = 0 ; i < str->length ; i++)
    printf("%2d  U+%04x   %c      %c      %c     %c      %c     %c      %2d       %2d        %2d         %2d\n",
	   i,
	   str->data[i],
	   (impulse_unichar_isascii(str->data[i]) && 
	    impulse_unichar_isprint(str->data[i])) ? str->data[i] : ' ',
	   impulse_unichar_iscontrol(str->data[i]) ? 'x' : ' ',
	   impulse_unichar_isalpha(str->data[i])? 'x' : ' ',
	   impulse_unichar_isnumber(str->data[i])? 'x' : ' ',
	   impulse_unichar_isspace(str->data[i])? 'x' : ' ',
	   impulse_unichar_ispunct(str->data[i])? 'x' : ' ',
	   impulse_unichar_category(str->data[i]),
	   impulse_unichar_direction(str->data[i]),
	   impulse_unichar_combining(str->data[i]),
	   impulse_unichar_formatting(str->data[i])
	   );

  impulse_unistring_destroy(str);
  impulse_unistring_destroy(encoding);
  impulse_cleanup();
  
  return 0;
}


/* End of file */
