/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * message.h
 * ---------
 *
 * Abstract message handling
 */


#ifndef __impulse_comms_message_h
#define __impulse_comms_message_h

#include <impulse/comms/transport.h>

/*                            ======--------======                           */

typedef struct _impulse_message_header *impulse_message_header;
typedef struct _impulse_message *impulse_message;
typedef struct _impulse_message_formatter *impulse_message_formatter;

typedef void (*impulse_message_writer)(impulse_message,
				       impulse_uint,
				       void *);
typedef void (*impulse_message_reader)(impulse_message,
				       impulse_uint,
				       void *);


struct _impulse_message_header
{
  impulse_ubyte service;
  impulse_ubyte opcode;
  impulse_ushort length;
};

struct _impulse_message
{
  struct _impulse_message_header header;
  void *payload;
  void *offset;
};


struct _impulse_message_formatter
{
  impulse_char key;
  impulse_message_writer writer;
  impulse_message_reader reader;
};


#define IMPULSE_MESSAGE_BYTE       'b'
#define IMPULSE_MESSAGE_UBYTE      'B'
#define IMPULSE_MESSAGE_SHORT      's'
#define IMPULSE_MESSAGE_USHORT     'S'
#define IMPULSE_MESSAGE_INT        'i'
#define IMPULSE_MESSAGE_UINT       'I'
#define IMPULSE_MESSAGE_LONG       'l'
#define IMPULSE_MESSAGE_ULONG      'L'
#define IMPULSE_MESSAGE_FLOAT      'F'
#define IMPULSE_MESSAGE_DOUBLE     'D'
#define IMPULSE_MESSAGE_CHAR       'c'
#define IMPULSE_MESSAGE_UNICHAR    'u'
#define IMPULSE_MESSAGE_BOOL       'o'
#define IMPULSE_MESSAGE_LIST       'x'


/*                            ======--------======                           */

impulse_message impulse_message_create(impulse_ubyte service,
				       impulse_ubyte opcode,
				       impulse_ushort length);
void impulse_message_destroy(impulse_message message);

#define impulse_message_write(m,f,args...) impulse_message_writef(m,0,NULL,f, ##args)
#define impulse_message_read(m,f,args...) impulse_message_readf(m,0,NULL,f, ##args)

void impulse_message_writef(impulse_message message,
			    impulse_uint nformatters,
			    impulse_message_formatter *formatters,
			    impulse_char *format, 
			    ...);
void impulse_message_readf(impulse_message message,
			   impulse_uint nformatters,
			   impulse_message_formatter *formatters,
			   impulse_char *format, 
			   ...);

#endif

/* End of file */
