/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * transport_sock_int.h
 * --------------------
 *
 * Communication transport functions
 * Sockets protocols
 */


#ifndef __impulse_comms_transport_sock_int_h
#define __impulse_comms_transport_sock_int_h

#include <impulse/comms/transport_sock.h>


#define IMPULSE_TRANSPORT_SOCK_TCP4 0  /* IPv4 stream protocol */
#define IMPULSE_TRANSPORT_SOCK_UDP4 1  /* IPv4 datagram protocol */
#define IMPULSE_TRANSPORT_SOCK_TCP6 2  /* IPv6 stream protocol */
#define IMPULSE_TRANSPORT_SOCK_UDP6 3  /* IPv6 datagram protocol */
#define IMPULSE_TRANSPORT_SOCK_TCPL 4  /* Local stream protocol */
#define IMPULSE_TRANSPORT_SOCK_UDPL 5  /* Local datagram protocol */

#define IMPULSE_TRANSPORT_SOCK_MAX IMPULSE_TRANSPORT_SOCK_UDPL + 1

struct _impulse_transport_sock
{
  impulse_int handle;
  impulse_byte protocol;

  impulse_transport_handler handler;
  void *edata;
};

typedef struct _impulse_transport_sock *impulse_transport_sock;

/*                            ======--------======                           */

void impulse_transport_sock_init(void);
void impulse_transport_sock_cleanup(void);

void impulse_transport_sock_create(impulse_transport transport);
void impulse_transport_sock_destroy(impulse_transport transport);

/* Event functions */
impulse_int impulse_transport_sock_handle(impulse_transport transport,
					  impulse_transport_handler func,
					  void *edata);
impulse_int impulse_transport_sock_unhandle(impulse_transport transport);


/* Name resolution stuff */

impulse_int impulse_transport_sock_lookup(impulse_transport transport,
					  impulse_unistring name,
					  impulse_unistring *address);
impulse_int impulse_transport_sock_rlookup(impulse_transport transport,
					   impulse_unistring address,
					   impulse_unistring *name);

/* Connection functions */
impulse_int impulse_transport_sock_open(impulse_transport transport,
					impulse_unistring local,
					impulse_unistring remote);
impulse_int impulse_transport_sock_accept(impulse_transport transport,
					  impulse_transport *client);
impulse_int impulse_transport_sock_close(impulse_transport transport);
impulse_int impulse_transport_sock_active(impulse_transport transport);

/* Data transfer functions */
impulse_int impulse_transport_sock_pending(impulse_transport transport);
impulse_int impulse_transport_sock_send(impulse_transport transport,
					impulse_byte *buffer,
					impulse_uint size,
					impulse_unistring address);
impulse_int impulse_transport_sock_receive(impulse_transport transport,
					   impulse_byte *buffer,
					   impulse_uint size,
					   impulse_unistring *address);

#endif

/* End of file */
