/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the ROWS widget library
 * for Impulse.
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * screen_xwin.c
 * -------------
 *
 * An implementation of drawables for drawing on screen
 */

#include "../../../config.h"

#include "impulse/rows/screen_xwin.h"
#include "impulse/util/memory.h"
#include "impulse/util/geometry.h"
#include "impulse/rows/main_xwin.h"
#include "impulse/rows/brush_int.h"
#include "impulse/rows/bitmap_int.h"
#include "impulse/rows/child_int.h"
#include "impulse/rows/child_xwin.h"

#ifdef HAVE_XEXT_MIT_SHM
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>
#endif

struct _p_ROWS_X_bitmap_data {
  /* Native image data */
  
  ROWS_bitmap  bitmap;
  XImage      *image;
  Pixmap       pixmap;

  int use_shm;

#ifdef HAVE_XEXT_MIT_SHM
  XShmSegmentInfo shminfo;
#endif

  impulse_uint mod_id;
};

/*                            ======--------======                           */
void ROWS_screen_destroy(void **widget);

void ROWS_screen_setup_int(ROWS_widget *widget,
			   ROWS_child child)
{
  _p_ROWS_X_screen_data   **priv;
  _p_ROWS_child_data   **child_priv;
  _p_ROWS_X_child_data  *child_data;
  child_priv = (_p_ROWS_child_data **) child;
    
  child_data = (*child_priv)->int_data;
  
  priv  = (_p_ROWS_X_screen_data **) ROWS_widget_private(widget, screen_ID);
  *priv = impulse_alloc(sizeof(_p_ROWS_X_screen_data));

  (*priv)->drawable = child_data->window;
  if (appl->visual)
    (*priv)->maingc = XCreateGC(appl->display,
				child_data->window,
				0,
				NULL);
  else
    (*priv)->maingc = 0;
  
  (*priv)->visual      = appl->visual;
  (*priv)->display     = appl->display;
  (*priv)->cmap        = appl->cmap;

  (*priv)->clip = XCreateRegion();
  (*priv)->regions     = NULL;

  (*priv)->rshift      = -1;

#ifdef HAVE_XEXT_MIT_SHM
  if (XShmQueryExtension((*priv)->display))
    {
      (*priv)->use_shm = 1;
      (*priv)->use_shm = 0;
    }
  else
    {
      (*priv)->use_shm = 0;
    }
#else
  (*priv)->use_shm = 0;
#endif

  if ((*priv)->cmap != None)
    ROWS_drawable_postinitialise(*widget, 1);
  else
    ROWS_drawable_postinitialise(*widget, 0);
  
  ROWS_widget_set_destructor(widget, screen_ID, ROWS_screen_destroy);
}


void ROWS_screen_destroy(void **widget)
{
  _p_ROWS_X_screen_data **priv = (_p_ROWS_X_screen_data **)widget;
  _p_ROWS_X_screen_region_stack current;
  
  XFreeGC(appl->display,
	  (*priv)->maingc);

  XDestroyRegion((*priv)->clip);
  
  current = (*priv)->regions;
  while (current)
  {
    _p_ROWS_X_screen_region_stack next = current->next;
    
    XDestroyRegion(current->clip);
    impulse_free(current);
    
    current = next;
  }
  impulse_free(priv);
}

			     
void ROWS_screen_brush_create(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data   **priv;
  _p_ROWS_X_brush_data  *data;

  priv = (_p_ROWS_brush_data **) param->brush.brush;
  data    = impulse_alloc(sizeof(_p_ROWS_X_brush_data));

  data->value_mask = GCLineStyle | GCLineWidth;
  data->GCvalues.line_style = LineSolid;
  data->GCvalues.line_width = 0;
  data->this_font  = None;
  data->use_dashes = 0;

  (*priv)->data = data;
}


void ROWS_screen_brush_destroy(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data   **priv;
  _p_ROWS_X_brush_data  *data;

  priv = (_p_ROWS_brush_data **) param->brush.brush;
  data    = (*priv)->data;

  if (data->use_dashes)
    {
      impulse_free(data->dashes);
    }

  impulse_free(data);
}


void _p_ROWS_screen_brush_set_gc(ROWS_brush brush)
{
  _p_ROWS_brush_data      **data;
  _p_ROWS_X_screen_data **drawable;
  _p_ROWS_drawable_data   **draw;

  data     = (_p_ROWS_brush_data **) brush;
  draw     = (_p_ROWS_drawable_data **) (*data)->drawable;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);

  if (!(*data)->use_private_gc && !((*draw)->last_brush == brush))
    {
      _p_ROWS_X_brush_data *Xdata;
      
      Xdata = (*data)->data;

      XChangeGC((*drawable)->display,
		(*drawable)->maingc,
		Xdata->value_mask,
		&Xdata->GCvalues);

      if (Xdata->use_dashes)
	{
	  XSetDashes((*drawable)->display,
		     (*drawable)->maingc,
		     Xdata->offset,
		     Xdata->dashes,
		     Xdata->n_dashes);
	}

      (*draw)->last_brush = brush;
      Xdata->private_gc     = (*drawable)->maingc;
    }

  if ((*data)->use_private_gc)
    {
      _p_ROWS_X_brush_data *Xdata;
      
      Xdata = (*data)->data;

      ROWS_widget_lock((void **) drawable);
      if ((*data)->last_serial != ROWS_drawable_get_clip_serial((void **) drawable))
	{
	  impulse_mutex_X_lock;
#if 0
	  if ((*draw)->use_region)
	    {
	      /*
	       * FIXME: it would be more efficient just to use the
	       * clipping Pixmap from the global GC here.
	       */
	      XSetRegion((*drawable)->display,
			 Xdata->private_gc,
			 (*drawable)->clip);
	    }
	  else
	    {
	      XSetClipRectangles((*drawable)->display,
				 Xdata->private_gc,
				 0, 0,
				 &(*drawable)->current_clip,
				 1, Unsorted);
	    }
	  
	  (*data)->last_serial = ROWS_drawable_get_clip_serial((void **) drawable);
#endif	    
	  impulse_mutex_X_unlock;
	}
      ROWS_widget_unlock((void **) drawable);
    }
}


void ROWS_screen_brush_use_private_gc(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data      **data;
  _p_ROWS_X_brush_data     *Xdata;
  _p_ROWS_X_screen_data **drawable;

  data     = (_p_ROWS_brush_data **) param->brush.brush;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);
  Xdata    = (*data)->data;

  impulse_mutex_X_lock;
  Xdata->private_gc = XCreateGC((*drawable)->display,
				  (*drawable)->drawable,
				  Xdata->value_mask,
				  &Xdata->GCvalues);
  (*data)->use_private_gc = 1;
  impulse_mutex_X_unlock;
}


void ROWS_screen_brush_set_gc(ROWS_drawable_fparam *param)
{
  _p_ROWS_screen_brush_set_gc(param->brush.brush);
}


/* This function assumes the brush is already locked */
void _p_ROWS_screen_brush_set_private_gc(ROWS_brush brush)
{
  _p_ROWS_brush_data      **data;
  _p_ROWS_X_screen_data **drawable;
  _p_ROWS_X_brush_data     *Xdata;

  data     = (_p_ROWS_brush_data **) brush;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);
  Xdata    = (*data)->data;

  impulse_mutex_X_lock;
  XChangeGC((*drawable)->display,
	    Xdata->private_gc,
	    Xdata->value_mask,
	    &Xdata->GCvalues);

  if (Xdata->use_dashes)
    {
      XSetDashes((*drawable)->display,
		 (*drawable)->maingc,
		 Xdata->offset,
		 Xdata->dashes,
		 Xdata->n_dashes);
    }
  impulse_mutex_X_unlock;
}

/*                            ======--------======                           */

/* Functions to manipulate brush attributes */

void ROWS_screen_brush_set_dotpattern(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data    **data;
  _p_ROWS_X_brush_data   *Xdata;
  ROWS_brush_dotpattern   next;
  
  microns curpos, expos;

  data  = (_p_ROWS_brush_data **) param->brush.brush;
  Xdata = (*data)->data;

  if (Xdata->use_dashes == 0)
    {
      Xdata->use_dashes = 1;
      Xdata->dashes = NULL;
    }

  curpos = expos = 0;

  next = param->brush.data.dotpattern.dp;

  Xdata->value_mask |= GCLineStyle;
  Xdata->GCvalues.line_style = LineOnOffDash;

  if (next == NULL)
    {
      Xdata->GCvalues.line_style = LineSolid;
      Xdata->use_dashes = 0;
      if (Xdata->dashes != NULL)
	impulse_free(Xdata->dashes);
    }
  Xdata->n_dashes = 0;

  while (next != NULL)
    {
      int evpix, odpix, st;

      evpix = MICRONS_TO_PIXEL(next->solid_len-(curpos-expos));
      if (evpix <= 0)
	evpix = 1;
      expos  += next->solid_len;
      curpos += PIXEL_TO_MICRONS(evpix);
 
      odpix = MICRONS_TO_PIXEL(next->empty_len-(curpos-expos));
      if (odpix <= 0)
	odpix = 1;
      expos  += next->empty_len;
      curpos += PIXEL_TO_MICRONS(odpix);

      st = Xdata->n_dashes;
      Xdata->n_dashes+=2;
      Xdata->dashes = impulse_realloc(Xdata->dashes,
				      sizeof(char)*Xdata->n_dashes);

      Xdata->dashes[st]   = evpix;
      Xdata->dashes[st+1] = odpix;
      
      next = next->next;
    }

  Xdata->offset = param->brush.data.dotpattern.offset;

  if ((*data)->use_private_gc)
    _p_ROWS_screen_brush_set_private_gc((ROWS_brush) data);
}

void ROWS_screen_brush_set_linewidth(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data    **data;
  _p_ROWS_X_brush_data   *Xdata;

  data  = (_p_ROWS_brush_data **) param->brush.brush;
  Xdata = (*data)->data;

  Xdata->value_mask          |= GCLineWidth;
  Xdata->GCvalues.line_width  = MICRONS_TO_PIXEL(param->brush.data.linewidth.width);

  if ((*data)->use_private_gc)
    _p_ROWS_screen_brush_set_private_gc((ROWS_brush) data);
}


void ROWS_screen_brush_get_fgpixel(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data **data;

  data = (_p_ROWS_brush_data **) param->brush.brush;

  param->brush.data.singleint.value = ((struct _ROWS_X_colour_data *)
				       ((*data)->colour->data))->realcolour.pixel;
}


void ROWS_screen_brush_set_fgcolour(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data         **data;
  struct _ROWS_X_colour_data  *col;
  _p_ROWS_X_brush_data        *Xdata;

  data  = (_p_ROWS_brush_data **) param->brush.brush;
  col   = param->brush.data.colour.colour->data;
  Xdata = (*data)->data;

  Xdata->value_mask          |= GCForeground;
  Xdata->GCvalues.foreground  = col->realcolour.pixel;
  (*data)->colour             = param->brush.data.colour.colour;

  if ((*data)->use_private_gc)
    _p_ROWS_screen_brush_set_private_gc(param->brush.brush);
}

void ROWS_screen_brush_set_bgcolour(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data         **data;
  struct _ROWS_X_colour_data  *col;
  _p_ROWS_X_brush_data        *Xdata;

  data  = (_p_ROWS_brush_data **) param->brush.brush;
  Xdata = (*data)->data;

  col = param->brush.data.colour.colour->data;

  Xdata->value_mask          |= GCForeground;
  Xdata->GCvalues.background  = col->realcolour.pixel;
  (*data)->colour             = param->brush.data.colour.colour;

  if ((*data)->use_private_gc)
    _p_ROWS_screen_brush_set_private_gc(param->brush.brush);
}

/*                            ======--------======                           */


/* Functions to draw with this brush */

void ROWS_screen_brush_draw_line(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data      **data;
  _p_ROWS_X_screen_data **drawable;
  _p_ROWS_X_brush_data     *Xdata;

  data     = (_p_ROWS_brush_data **)    param->brush.brush;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);
  Xdata    = (*data)->data;

  impulse_mutex_X_lock;
  _p_ROWS_screen_brush_set_gc(param->brush.brush);

  XDrawLine((*drawable)->display,
	    (*drawable)->drawable,
	    Xdata->private_gc,
	    MICRONS_TO_PIXEL(param->brush.data.line.x1),
	    MICRONS_TO_PIXEL(param->brush.data.line.y1),
	    MICRONS_TO_PIXEL(param->brush.data.line.x2),
	    MICRONS_TO_PIXEL(param->brush.data.line.y2));
  impulse_mutex_X_unlock;
}


void ROWS_screen_brush_draw_point(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data      **data;
  _p_ROWS_X_screen_data **drawable;
  _p_ROWS_X_brush_data     *Xdata;

  data     = (_p_ROWS_brush_data **)    param->brush.brush;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);
  Xdata    = (*data)->data;

  impulse_mutex_X_lock;
  _p_ROWS_screen_brush_set_gc(param->brush.brush);

  XDrawPoint((*drawable)->display,
	     (*drawable)->drawable,
	     Xdata->private_gc,
	     MICRONS_TO_PIXEL(param->brush.data.coordinate.x),
	     MICRONS_TO_PIXEL(param->brush.data.coordinate.y));
  impulse_mutex_X_unlock;
}


void ROWS_screen_brush_draw_rectangle(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data      **data;
  _p_ROWS_X_screen_data **drawable;
  _p_ROWS_X_brush_data     *Xdata;

  data     = (_p_ROWS_brush_data **)    param->brush.brush;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);
  Xdata    = (*data)->data;

  impulse_mutex_X_lock;
  _p_ROWS_screen_brush_set_gc(param->brush.brush);

  XDrawRectangle((*drawable)->display,
		 (*drawable)->drawable,
		 Xdata->private_gc,
		 MICRONS_TO_PIXEL(param->brush.data.line.x1),
		 MICRONS_TO_PIXEL(param->brush.data.line.y1),
		 MICRONS_TO_PIXEL(param->brush.data.line.x2)-1,
		 MICRONS_TO_PIXEL(param->brush.data.line.y2)-1);
  impulse_mutex_X_unlock;
}


void ROWS_screen_brush_draw_arc(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data      **data;
  _p_ROWS_X_screen_data **drawable;
  _p_ROWS_X_brush_data     *Xdata;

  data     = (_p_ROWS_brush_data **)    param->brush.brush;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);
  Xdata    = (*data)->data;

  impulse_mutex_X_lock;
  _p_ROWS_screen_brush_set_gc(param->brush.brush);

  XDrawArc((*drawable)->display,
	   (*drawable)->drawable,
	   Xdata->private_gc,
	   MICRONS_TO_PIXEL(param->brush.data.arc.x),
	   MICRONS_TO_PIXEL(param->brush.data.arc.y),
	   MICRONS_TO_PIXEL(param->brush.data.arc.w),
	   MICRONS_TO_PIXEL(param->brush.data.arc.h),
	   param->brush.data.arc.angle1, param->brush.data.arc.angle2);
  impulse_mutex_X_unlock;
}


void ROWS_screen_brush_fill_rectangle(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data      **data;
  _p_ROWS_X_screen_data **drawable;
  _p_ROWS_X_brush_data     *Xdata;

  data     = (_p_ROWS_brush_data **)    param->brush.brush;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);
  Xdata    = (*data)->data;

  impulse_mutex_X_lock;
  _p_ROWS_screen_brush_set_gc(param->brush.brush);

  XFillRectangle((*drawable)->display,
		 (*drawable)->drawable,
		 Xdata->private_gc,
		 MICRONS_TO_PIXEL(param->brush.data.line.x1),
		 MICRONS_TO_PIXEL(param->brush.data.line.y1),
		 MICRONS_TO_PIXEL(param->brush.data.line.x2),
		 MICRONS_TO_PIXEL(param->brush.data.line.y2));
  impulse_mutex_X_unlock;
}


void ROWS_screen_brush_fill_arc(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data      **data;
  _p_ROWS_X_screen_data **drawable;
  _p_ROWS_X_brush_data     *Xdata;

  data     = (_p_ROWS_brush_data **)    param->brush.brush;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);
  Xdata    = (*data)->data;

  impulse_mutex_X_lock;
  _p_ROWS_screen_brush_set_gc(param->brush.brush);

  XFillArc((*drawable)->display,
	   (*drawable)->drawable,
	   Xdata->private_gc,
	   MICRONS_TO_PIXEL(param->brush.data.arc.x),
	   MICRONS_TO_PIXEL(param->brush.data.arc.y),
	   MICRONS_TO_PIXEL(param->brush.data.arc.w),
	   MICRONS_TO_PIXEL(param->brush.data.arc.h),
	   param->brush.data.arc.angle1, param->brush.data.arc.angle2);
  impulse_mutex_X_unlock;
}

/*                            ======--------======                           */

/* X colour functions */

extern _p_ROWS_drawable_data **_p_ROWS_drawable_get_data(ROWS_drawable widget);

static XColor *_p_ROWS_X_nearest_colour(int r, int g, int b,
					Display   *display,
					Visual    *visual,
					Colormap   cmap)
{
  static XVisualInfo *newinfo    = NULL;
  static XColor      *colourlist = NULL;
  static Visual      *lastvisual = NULL;
  static int          ncols      = 0;

  int                 x;
  float               redout, greenout, blueout;
  float               minout, totalout;
  int                 chosen;

  if (visual != lastvisual)
    {
      XVisualInfo info;
      int         nitems;
     
      impulse_mutex_X_lock;

      if (newinfo != NULL)
	XFree(newinfo);

      info.visual   = visual;
      info.visualid = XVisualIDFromVisual(visual);
      
      newinfo = XGetVisualInfo(display,
			       VisualIDMask,
			       &info,
			       &nitems);

      impulse_mutex_X_unlock;

      if (!newinfo)
	ROWS_error_raise(IMPULSE_ERROR_CRITICAL, ROWS_ERROR_INTERNAL, 0,
			 "Unable to get visual information while allocating colours");

      ncols      = newinfo->colormap_size;
      colourlist = impulse_realloc(colourlist, ncols*sizeof(XColor));

      for (x=0;x<ncols;x++)
	colourlist[x].pixel = x;
    }

  impulse_mutex_X_lock;
  XQueryColors(display, cmap, colourlist, ncols);
  impulse_mutex_X_unlock;

  r *= 256;
  g *= 256;
  b *= 256;

  chosen = -1;
  minout = 3.0;
  
  for (x=0;x<ncols;x++)
    {
      redout   = ((float) (colourlist[x].red   - r))/65536.0;
      greenout = ((float) (colourlist[x].green - g))/65536.0;
      blueout  = ((float) (colourlist[x].blue  - b))/65536.0;

      if (redout   < 0) redout   = -redout;
      if (greenout < 0) greenout = -greenout;
      if (blueout  < 0) blueout  = -blueout;

      totalout = redout+greenout+blueout;

      if (totalout<minout)
	{
	  chosen = x;
	  minout = totalout;
	}

      if (minout == 0)
	return &colourlist[chosen];
    }

  if (chosen == -1)
    ROWS_error_raise(IMPULSE_ERROR_CRITICAL, ROWS_ERROR_INTERNAL, 0,
		     "Unable to allocate colours");
  
  if (minout>0.25)
    ROWS_error_raise(IMPULSE_ERROR_WARNING, ROWS_ERROR_INTERNAL, 0,
		      "Forced to select a badly-matching colour (colourmap full)");
  
  return &colourlist[chosen];
}


void _p_ROWS_X_colour_allocate(struct _ROWS_X_colour_data *col)
{
  col->realcolour.red   = col->r*16;
  col->realcolour.green = col->g*16;
  col->realcolour.blue  = col->b*16;
  col->realcolour.flags = DoRed | DoGreen | DoBlue;

  if (!XAllocColor(col->xdata.display, col->xdata.cmap, &col->realcolour))
    {
#ifdef ROWS_DEBUG
      fprintf(stderr, "*** ROWSdebug: Grabbing X-server ***\n");
#endif
      impulse_mutex_X_lock;
      XGrabServer(col->xdata.display);
      impulse_mutex_X_unlock;

      col->realcolour = *_p_ROWS_X_nearest_colour(col->r, col->g, col->b,
						  col->xdata.display,
						  col->xdata.visual,
						  col->xdata.cmap);
      
      impulse_mutex_X_lock;
      if (!XAllocColor(col->xdata.display, col->xdata.cmap, &col->realcolour))
	{
	  XUngrabServer(col->xdata.display);
	  impulse_mutex_X_unlock;
	  ROWS_error_raise(IMPULSE_ERROR_CRITICAL, ROWS_ERROR_INTERNAL, 0,
			   "Unable to allocate colours");
	}

      XUngrabServer(col->xdata.display);
      impulse_mutex_X_unlock;
#ifdef ROWS_DEBUG
      fprintf(stderr, "*** ROWSdebug: X-server released ***\n");
#endif
    }
}

void ROWS_screen_colour_create(ROWS_drawable_fparam *param)
{
  struct _ROWS_X_colour_data *col;
  _p_ROWS_X_screen_data **priv;

#if USE_THREADS
  impulse_mutex_lock(appl->colourmap_lock);
#endif
  
  priv = (_p_ROWS_X_screen_data **) ROWS_screen_cast(param->colour.drawable);

  col = impulse_alloc(sizeof(struct _ROWS_X_colour_data));

  col->r = param->colour.data.create.rgb.data.rgb.r;
  col->g = param->colour.data.create.rgb.data.rgb.g;
  col->b = param->colour.data.create.rgb.data.rgb.b;
  
  col->xdata.display = (*priv)->display;
  col->xdata.cmap    = (*priv)->cmap;
  col->xdata.visual  = (*priv)->visual;

  col->was_pixel = 0;

  _p_ROWS_X_colour_allocate(col);

#if USE_THREADS
  impulse_mutex_unlock(appl->colourmap_lock);
#endif
  
  param->colour.data.create.data = col;
}

void ROWS_screen_colour_discard(ROWS_drawable_fparam *param)
{
  impulse_int colarray[1];
  struct _ROWS_X_colour_data *col;
  
#if USE_THREADS
  impulse_mutex_unlock(appl->colourmap_lock);
#endif
  
  col = param->colour.data.discard.data;
  colarray[0] = col->realcolour.pixel;

  if (!col->was_pixel)
    {
      XFreeColors(col->xdata.display,
		  col->xdata.cmap,
		  (unsigned long *)colarray,
		  1,
		  0);
    }
  
  impulse_free(param->colour.data.discard.data);
  
#if USE_THREADS
  impulse_mutex_unlock(appl->colourmap_lock);
#endif  
}

void ROWS_screen_colour_pixel(ROWS_drawable_fparam *param)
{
  struct _ROWS_X_colour_data *col;
  _p_ROWS_X_screen_data **priv;

#if USE_THREADS
  impulse_mutex_lock(appl->colourmap_lock);
#endif
  
  priv = (_p_ROWS_X_screen_data **) ROWS_screen_cast(param->colour.drawable);

  col = impulse_alloc(sizeof(struct _ROWS_X_colour_data));

  col->r = -1;
  col->g = -1;
  col->b = -1;

  col->xdata.display = (*priv)->display;
  col->xdata.cmap    = (*priv)->cmap;
  col->xdata.visual  = (*priv)->visual;

  col->was_pixel = 1;

  col->realcolour.red   = -1;
  col->realcolour.green = -1;
  col->realcolour.blue  = -1;
  col->realcolour.flags = 0;
  col->realcolour.pixel = param->colour.data.create_pixel.pixel;
  
#if USE_THREADS
  impulse_mutex_unlock(appl->colourmap_lock);
#endif
  
  param->colour.data.create.data = col;
}

/*                            ======--------======                           */

/* X Drawable functions */

void ROWS_screen_clear_area(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data **priv;

  priv = (_p_ROWS_X_screen_data **) ROWS_screen_cast(param->drawable.drawable);

  impulse_mutex_X_lock;

  XClearArea((*priv)->display,
	     (*priv)->drawable,
	     MICRONS_TO_PIXEL(param->drawable.data.clear_area.x),
	     MICRONS_TO_PIXEL(param->drawable.data.clear_area.y),
	     MICRONS_TO_PIXEL(param->drawable.data.clear_area.w),
	     MICRONS_TO_PIXEL(param->drawable.data.clear_area.h),
	     True);

  impulse_mutex_X_unlock;
}

void ROWS_screen_sync(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data **priv;

  priv = (_p_ROWS_X_screen_data **) ROWS_screen_cast(param->drawable.drawable);

  impulse_mutex_X_lock;

  XFlush((*priv)->display);

  impulse_mutex_X_unlock;
}

static void _p_ROWS_X_create_bitmap_data(void *data)
{
  struct _p_ROWS_X_bitmap_data *x_data;

  x_data = data;
  x_data->bitmap = NULL;
  x_data->image  = NULL;
  x_data->pixmap = None;

  x_data->mod_id = 0xffffffff;
}

static void _p_ROWS_X_destroy_bitmap_data(void *data)
{
  struct _p_ROWS_X_bitmap_data *x_data;

  x_data = data;
}

static impulse_byte nocorrect_16[] = { 
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
  12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 
  22, 23, 24, 25, 26, 27, 28, 29, 30, 31 
};

static inline int getshift(impulse_uint mask)
{
  int firstbit, lastbit, x;

  firstbit = -1;
  lastbit  = -1;
  for (x=0; x<32; x++)
    {
      if ((mask&(1<<x)) != 0)
	{
	  firstbit = x;
	  x = 32;
	}
    }

  if (firstbit == -1)
    return -1;

  for (x=firstbit; x<32; x++)
    {
      if ((mask&(1<<x)) == 0)
	{
	  lastbit = x;
	  x = 32;
	}
    }

  if (lastbit == -1)
    {
      lastbit = 31;
    }

  return lastbit;
}

void ROWS_screen_draw_bitmap(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data      **priv;
  struct _p_ROWS_X_bitmap_data  *x_data;
  _p_ROWS_bitmap_data          **bit_data;
  
  priv  = (_p_ROWS_X_screen_data **)
    ROWS_screen_cast(param->drawable.drawable);

  /* FIXME: we do not need to create new images for the same display */
  x_data =
    _p_ROWS_bitmap_get_private_data(param->drawable.data.draw_bitmap.bitmap,
				    param->drawable.drawable,
				    sizeof(struct _p_ROWS_X_bitmap_data),
				    _p_ROWS_X_create_bitmap_data,
				    _p_ROWS_X_destroy_bitmap_data);
  bit_data = (_p_ROWS_bitmap_data **)
    param->drawable.data.draw_bitmap.bitmap;

  /* Get rid of the old data if the bitmap has changed */
  if ((*bit_data)->mod_id != x_data->mod_id)
    {
      if (x_data->pixmap != None)
	XFreePixmap((*priv)->display,
		    x_data->pixmap);
      if (x_data->bitmap != NULL)
	ROWS_widget_destroy(x_data->bitmap);
      if (x_data->image != NULL)
	{
#ifdef HAVE_XEXT_MIT_SHM
	  if (!x_data->use_shm)
	    {
	      XDestroyImage(x_data->image);
	    }
	  else
	    {
	      XShmDetach((*priv)->display, &x_data->shminfo);
	    }
#else
	  XDestroyImage(x_data->image);
#endif
	}

      x_data->pixmap = None;
      x_data->bitmap = NULL;
      x_data->image  = NULL;
      x_data->mod_id = (*bit_data)->mod_id;
    }
  
  if (x_data->pixmap == None)
    {
      /* Hmm, need to create an image */
      if (x_data->image == NULL)
	{
	  if (x_data->bitmap == NULL)
	    {
	      int depth;
	      _p_ROWS_bitmap_data **x_bit_data;
 
	      /*
	       * We assume we're using the display depth here, as X
	       * doesn't provide a function for finding the depth of a 
	       * drawable (or a visual, for that matter)
	       */
	      depth = DisplayPlanes((*priv)->display, 0);

	      if (depth < 16)
		{
		  impulse_unistring desc = impulse_unistring_create_utf("Cannot plot bitmaps on depths < 16", 8);
		  impulse_error_raise(IMPULSE_ERROR_WARNING, desc, NULL);
		  impulse_unistring_destroy(desc);
		}

	      if (depth>=16 && (*bit_data)->format >=16)
		{
		  XImage *image;
		  int     rshift1, rshift2;
		  int     gshift1, gshift2;
		  int     bshift1, bshift2;
		  int     x,y,nbytes;
		  
		  /*
		   * Truecolor bitmaps on Truecolor displays are easy
		   * to cope with
		   */

		  /* FIXME: deal with lowcolor bitmaps as well... */
		  /* FIXME: what about directcolor? */

		  /*
		   * Note: X guarantees that the format of a pixel
		   * will be RRRGGGBBB or similar, not RGBRGBRGB (ie
		   * all colour bits together). So, we can work out
		   * the shifts required to create the image from the
		   * XImage structure.
		   */

#ifndef HAVE_XEXT_MIT_SHM
		  image = XCreateImage((*priv)->display,
				       (*priv)->visual,
				       depth,
				       ZPixmap,
				       0,
				       NULL,
				       (*bit_data)->width,
				       (*bit_data)->height,
				       32,
				       0);
#else
		  if ((*priv)->use_shm || 0)
		    {
		      image = XShmCreateImage((*priv)->display,
					      (*priv)->visual,
					      depth,
					      ZPixmap,
					      NULL,
					      &x_data->shminfo,
					      (*bit_data)->width,
					      (*bit_data)->height);
		      x_data->shminfo.shmid =
			shmget(IPC_PRIVATE,
			       image->bytes_per_line*image->height,
			       IPC_CREAT|0777);
		      x_data->shminfo.shmaddr =
			image->data =
			shmat(x_data->shminfo.shmid, 0, 0);
		      XShmAttach((*priv)->display,
				 &x_data->shminfo);

		      x_data->use_shm = 1;
		    }
		  else
		    {
		      image = XCreateImage((*priv)->display,
					   (*priv)->visual,
					   depth,
					   ZPixmap,
					   0,
					   NULL,
					   (*bit_data)->width,
					   (*bit_data)->height,
					   32,
					   0);

		      x_data->use_shm = 0;
		    }
#endif

		  if (image->bits_per_pixel != 16 &&
		      image->bits_per_pixel != 32)
		    {
		      /*
		       * Actually, the conversion code would handle a
		       * 24bpp screen just fine
		       */
		      impulse_unistring desc = impulse_unistring_create_utf("Cannot plot bitmap -True/Directcolor images must be word-aligned", 8);
		      impulse_error_raise(IMPULSE_ERROR_CRITICAL, desc, NULL);
		      impulse_unistring_destroy(desc);
		    }

		  /* Work out shift values to get correct pixels */
		  if ((*priv)->rshift == -1)
		    {
		      (*priv)->rshift = getshift(image->red_mask);
		      (*priv)->gshift = getshift(image->green_mask);
		      (*priv)->bshift = getshift(image->blue_mask);
		    }

		  rshift1 = gshift1 = bshift1 = 0;
		  
		  if ((*bit_data)->format == 32)
		    {
		      rshift2 = (*priv)->rshift-8;
		      gshift2 = (*priv)->gshift-16;
		      bshift2 = (*priv)->bshift-24;
		    }
		  else
		    {
		      rshift2 = (*priv)->rshift-5;
		      gshift2 = (*priv)->gshift-10;
		      bshift2 = (*priv)->bshift-15;
		    }

		  rshift1 = (rshift2<0)?-rshift2:0;
		  gshift1 = (gshift2<0)?-gshift2:0;
		  bshift1 = (bshift2<0)?-bshift2:0;

		  rshift2 = (rshift2>0)?rshift2:0;
		  gshift2 = (gshift2>0)?gshift2:0;
		  bshift2 = (bshift2>0)?bshift2:0;
		  
		  /* Allocate memory for the image */
		  image->data =
		    malloc(image->bytes_per_line * image->height);

		  nbytes = image->bits_per_pixel/8;

		  /* Perform the conversion */
		  for (y=0; y<(*bit_data)->height; y++)
		    {
		      int pos, bpos, pixel, byte;

		      /* Line iterator */

		      /* Work out the position in the image */
		      pos  = y*image->bytes_per_line;
		      bpos = y*(*bit_data)->lineskip;
		      
		      for (x=0; x<(*bit_data)->width; )
			{
			  impulse_uint value;

			  value = (*bit_data)->data[bpos];
			  
			  for (pixel=0; pixel<32;
			       pixel+=(*bit_data)->format)
			    {
			      impulse_uint newpixel;

			      newpixel =
				(((value<<rshift2)>>rshift1)&image->red_mask)|
				(((value<<gshift2)>>gshift1)&image->green_mask)|
				(((value<<bshift2)>>bshift1)&image->blue_mask);
			      
			      value >>= (*bit_data)->format;

			      for (byte=0; byte<nbytes; byte++)
				{
				  image->data[pos] = (char) newpixel;
				  newpixel >>= 8;
				  pos++;
				}
			      
			      x++;
			    }

			  bpos++;
			}
		    }

		  /* Store the image in the x_data structure */
		  x_data->image = image;
		}
	      else
		{
		  /* FIXME: utterly broken */
		  impulse_unistring desc = impulse_unistring_create_utf("Sorry lowcolour handling is buggered", 8);
		  impulse_error_raise(IMPULSE_ERROR_WARNING, desc, NULL);
		  impulse_unistring_destroy(desc);

		  x_data->bitmap =
		    ROWS_bitmap_convert(param->drawable.data.draw_bitmap.bitmap,
					depth,
					nocorrect_16);
		  x_bit_data = (_p_ROWS_bitmap_data **)
		    ROWS_bitmap_cast(x_data->bitmap);
		  
		  x_data->image = XCreateImage((*priv)->display,
					       (*priv)->visual,
					       depth,
					       ZPixmap,
					       0,
					       (char *)(*x_bit_data)->data,
					       (*x_bit_data)->width,
					       (*x_bit_data)->height,
					       32,
					       (*x_bit_data)->lineskip*4);
		  
		}
	    }
	}

      /* Load the image into a pixmap */

#ifdef HAVE_XEXT_MIT_SHM
      if (!x_data->use_shm)
	{
	  x_data->pixmap = XCreatePixmap((*priv)->display,
					 (*priv)->drawable,
					 (*bit_data)->width,
					 (*bit_data)->height,
					 DisplayPlanes((*priv)->display,
						       0));
	  XPutImage((*priv)->display,
		    x_data->pixmap,
		    (*priv)->maingc,
		    x_data->image,
		    0,0,
		    0,0,
		    (*bit_data)->width,
		    (*bit_data)->height);
	}
#else
      /* FIXME: check for errors (very likely) */
      x_data->pixmap = XCreatePixmap((*priv)->display,
				     (*priv)->drawable,
				     (*bit_data)->width,
				     (*bit_data)->height,
				     DisplayPlanes((*priv)->display,
						   0));
      XPutImage((*priv)->display,
		x_data->pixmap,
		(*priv)->maingc,
		x_data->image,
		0,0,
		0,0,
		(*bit_data)->width,
		(*bit_data)->height);
#endif
    }

  if (x_data->pixmap == None)
    {
#if HAVE_XEXT_MIT_SHM
      if (x_data->use_shm)
	{
	  /* FIXME */
	  XShmPutImage((*priv)->display,
		       (*priv)->drawable,
		       (*priv)->maingc,
		       x_data->image,
		       0,0,
		       MICRONS_TO_PIXEL(param->drawable.data.draw_bitmap.x),
		       MICRONS_TO_PIXEL(param->drawable.data.draw_bitmap.y),
		       (*bit_data)->width,(*bit_data)->height,
		       False);
	}
      else
#endif
	{
	  /* Put the image directly */
	  XPutImage((*priv)->display,
		    (*priv)->drawable,
		    (*priv)->maingc,
		    x_data->image,
		    0,0,
		    MICRONS_TO_PIXEL(param->drawable.data.draw_bitmap.x),
		    MICRONS_TO_PIXEL(param->drawable.data.draw_bitmap.y),
		    (*bit_data)->width,(*bit_data)->height);
	}
    }
  else
    {
      /* Put the image using a pixmap */
      XCopyArea((*priv)->display,
		x_data->pixmap,
		(*priv)->drawable,
		(*priv)->maingc,
		0,0,
		(*bit_data)->width, (*bit_data)->height,
		MICRONS_TO_PIXEL(param->drawable.data.draw_bitmap.x),
		MICRONS_TO_PIXEL(param->drawable.data.draw_bitmap.y));
    }
}


void ROWS_screen_clip_push(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data         **priv;
  _p_ROWS_X_screen_region_stack temp;
  _p_ROWS_drawable_data           **draw;

  priv = (_p_ROWS_X_screen_data **)
    ROWS_screen_cast(param->drawable.drawable);
  draw    = (_p_ROWS_drawable_data **) param->drawable.drawable;

  temp = impulse_alloc(sizeof(struct _p_ROWS_X_screen_region_stack));

  temp->clip = XCreateRegion();
  XUnionRegion((*priv)->clip,
	       temp->clip,
	       temp->clip);

  temp->next = (*priv)->regions;
  (*priv)->regions = temp;
}


void ROWS_screen_clip_pop(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data         **priv;
  _p_ROWS_X_screen_region_stack next;
  _p_ROWS_drawable_data       **draw;

  priv = (_p_ROWS_X_screen_data **)
    ROWS_screen_cast(param->drawable.drawable);
  draw    = (_p_ROWS_drawable_data **) param->drawable.drawable;

  if (!(*priv)->regions)
    return;

  XDestroyRegion((*priv)->clip);
  (*priv)->clip = (*priv)->regions->clip;
  next = (*priv)->regions->next;
  impulse_free((*priv)->regions);
  (*priv)->regions = next;
}


void ROWS_screen_clip_set(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data **priv;

  priv  = (_p_ROWS_X_screen_data **) ROWS_screen_cast(param->drawable.drawable);

  XSetRegion((*priv)->display, 
	     (*priv)->maingc, 
	     (*priv)->clip);
}


void ROWS_screen_clip_clear(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data **priv;

  priv  = (_p_ROWS_X_screen_data **)ROWS_screen_cast(param->drawable.drawable);

  XDestroyRegion((*priv)->clip);
  (*priv)->clip = XCreateRegion();
}


void ROWS_screen_clip_union(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data **priv;
  XRectangle              rect;

  priv  = (_p_ROWS_X_screen_data **) ROWS_screen_cast(param->drawable.drawable);

  rect.x      = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.pos.x);
  rect.y      = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.pos.y);
  rect.width  = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.size.x);
  rect.height = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.size.y);
  
  XUnionRectWithRegion(&rect, 
		       (*priv)->clip,
		       (*priv)->clip);
}


void ROWS_screen_clip_intersect(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data **priv;
  XRectangle rect;
  Region temp;

  priv  = (_p_ROWS_X_screen_data **) ROWS_screen_cast(param->drawable.drawable);

  temp = XCreateRegion();
  rect.x      = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.pos.x);
  rect.y      = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.pos.y);
  rect.width  = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.size.x);
  rect.height = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.size.y);
  XUnionRectWithRegion(&rect,
		       temp,
		       temp);
  
  XIntersectRegion(temp,
		   (*priv)->clip,
		   (*priv)->clip);
  XDestroyRegion(temp);
}


void ROWS_screen_clip_subtract(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data **priv;
  XRectangle rect;
  Region temp;
  
  priv  = (_p_ROWS_X_screen_data **) ROWS_screen_cast(param->drawable.drawable);

  temp = XCreateRegion();
  rect.x      = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.pos.x);
  rect.y      = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.pos.y);
  rect.width  = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.size.x);
  rect.height = MICRONS_TO_PIXEL(param->drawable.data.clip.rect.size.y);
    
  XUnionRectWithRegion(&rect,
		       temp,
		       temp);
  XSubtractRegion(temp,
		  (*priv)->clip,
		  (*priv)->clip);
  XDestroyRegion(temp);
}


void ROWS_screen_clip_bounds(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data **priv;
  XRectangle              rectangle;

  priv  = (_p_ROWS_X_screen_data **) ROWS_screen_cast(param->drawable.drawable);
  
  XClipBox((*priv)->clip,
	   &rectangle);

  param->drawable.data.clip.rect.pos.x  =
    PIXEL_TO_MICRONS(rectangle.x);
  param->drawable.data.clip.rect.pos.y  =
    PIXEL_TO_MICRONS(rectangle.y);
  param->drawable.data.clip.rect.size.x =
    PIXEL_TO_MICRONS(rectangle.width);
  param->drawable.data.clip.rect.size.y =
    PIXEL_TO_MICRONS(rectangle.height);
}


void ROWS_screen_clip_copy(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data **priv, **priv_from;
  Region emptyRegion;

  priv = (_p_ROWS_X_screen_data **)
    ROWS_screen_cast(param->drawable.drawable);
  priv_from = (_p_ROWS_X_screen_data **)
    ROWS_screen_cast(param->drawable.data.copy.from);

  XDestroyRegion((*priv)->clip);
  (*priv)->clip = XCreateRegion();
  emptyRegion             = XCreateRegion();
  
  XUnionRegion(emptyRegion,
	       (*priv_from)->clip,
	       (*priv)->clip);
  
  XOffsetRegion((*priv)->clip,
		MICRONS_TO_PIXEL(param->drawable.data.copy.offset.x),
		MICRONS_TO_PIXEL(param->drawable.data.copy.offset.y));
  
  XDestroyRegion(emptyRegion);

  ROWS_drawable_clip_set(param->drawable.drawable);
}


void ROWS_screen_clip_move(ROWS_drawable_fparam *param)
{
  _p_ROWS_X_screen_data **priv;

  priv = (_p_ROWS_X_screen_data **)
    ROWS_screen_cast(param->drawable.drawable);
  
  XOffsetRegion((*priv)->clip,
		MICRONS_TO_PIXEL(param->drawable.data.copy.offset.x),
		MICRONS_TO_PIXEL(param->drawable.data.copy.offset.y));
}

/*                            ======--------======                           */

/* Fonts */

/* FIXME: adapt for impulse_font as well as X fonts */

/* FIXME: need a way of getting font names */

struct _X_font_alloc {
  Font font;
};

void ROWS_screen_brush_set_font(ROWS_drawable_fparam *param)
{
  struct _X_font_alloc  *fontinfo;
  char                 **fontlist;
  int                    n_fonts;
  _p_ROWS_brush_data   **data;
  _p_ROWS_X_brush_data  *Xdata;
  impulse_unistring fontname;

  data  = (_p_ROWS_brush_data **) param->brush.brush;
  Xdata = (*data)->data;

  fontlist = XListFonts(appl->display,
			param->brush.data.set_font.font_name,
			1,
			&n_fonts);
  
  if (n_fonts <= 0)
    {
      /* No such font */
      return;
    }
#if USE_THREADS
  impulse_mutex_lock(appl->fonts_lock);
#endif
  
  fontname = impulse_unistring_create_utf(fontlist[0], 8);
  fontinfo = impulse_hash_recover(appl->fonts,
				  fontname);
  if (!fontinfo)
    {
      fontinfo = impulse_alloc(sizeof(struct _X_font_alloc));
      fontinfo->font = XLoadFont(appl->display,
				 fontlist[0]);

      impulse_hash_store(appl->fonts,
			 fontname,
			 fontinfo);
    }
  impulse_unistring_destroy(fontname);
  
#if USE_THREADS
  impulse_mutex_unlock(appl->fonts_lock);
#endif
  Xdata->value_mask    |= GCFont;
  Xdata->GCvalues.font  = fontinfo->font;
  Xdata->this_font      = fontinfo->font;
  
  if ((*data)->use_private_gc)
    _p_ROWS_screen_brush_set_private_gc(param->brush.brush);

  XFreeFontNames(fontlist);
}

void ROWS_screen_brush_set_font_size(ROWS_drawable_fparam *param)
{
  /* FIXME: Implement correctly for X fonts */
  return;
}

void ROWS_screen_brush_font_paint(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data      **data;
  _p_ROWS_X_screen_data **drawable;
  _p_ROWS_X_brush_data     *Xdata;
  char                     *realtext;
  
  data     = (_p_ROWS_brush_data **)    param->brush.brush;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);
  Xdata    = (*data)->data;

  _p_ROWS_screen_brush_set_gc(param->brush.brush);

  realtext = impulse_unistring_to_utf(param->brush.data.font_paint.text, 8,0);

  XDrawString((*drawable)->display,
	      (*drawable)->drawable,
	      Xdata->private_gc,
	      MICRONS_TO_PIXEL(param->brush.data.font_paint.x),
	      MICRONS_TO_PIXEL(param->brush.data.font_paint.y),
	      realtext,
	      strlen(realtext));

  impulse_free(realtext);
}

void ROWS_screen_brush_font_measure(ROWS_drawable_fparam *param)
{
   _p_ROWS_brush_data      **data;
  _p_ROWS_X_screen_data **drawable;
  _p_ROWS_X_brush_data     *Xdata;
  char                     *realtext;
  int                       direction, ascent, descent;
  XCharStruct               overall;
  ROWS_brush_measure       *measure;

  data     = (_p_ROWS_brush_data **)    param->brush.brush;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);
  Xdata    = (*data)->data;

  _p_ROWS_screen_brush_set_gc(param->brush.brush);

  realtext = impulse_unistring_to_utf(param->brush.data.font_measure.text, 8,0);

  /* FIXME: add a mechanism to use XTextExtents as well */
  XQueryTextExtents((*drawable)->display,
		    Xdata->this_font,
		    realtext,
		    strlen(realtext),
		    &direction, &ascent, &descent,
		    &overall);

  /* CHECKME: are these calculations correct? */
  measure = param->brush.data.font_measure.result;

  measure->width           = PIXEL_TO_MICRONS(overall.width);
  measure->ascend          = PIXEL_TO_MICRONS(ascent);
  measure->descend         = PIXEL_TO_MICRONS(descent);
  measure->left_overspill  = PIXEL_TO_MICRONS(overall.lbearing);
  measure->right_overspill = PIXEL_TO_MICRONS(overall.rbearing -
					      overall.width);
  
  impulse_free(realtext);
}

/*                            ======--------======                           */

/* Shape drawing */

static void make_bezier(impulse_geom_bezier  *curve,
			XPoint              **points,
			int                  *npoints)
{
  double delta;
  int n, x, s, e;
  double cx, cy;
  impulse_geom_cubic_delta d;

  /* Hmm, now to work out delta */

  /*
   * Ideally, we would work out how long the curve is (in pixels) and 
   * and use that to guess a good value... But that turns out to be a
   * *hard* thing to do, so the following will have to do...
   */

  /*
   * Sort-of the length (well it is if the line is horizontal or
   * vertical...)
   */
  delta =
    (curve->p[3].x>curve->p[0].x?
    (curve->p[3].x - curve->p[0].x):
    (curve->p[0].x - curve->p[3].x)) +
    (curve->p[3].y>curve->p[0].y?
    (curve->p[3].y - curve->p[0].y):
    (curve->p[0].y - curve->p[3].y));
  delta /= 2;

  if (delta < 4)
    delta = 4;
  n = delta;
  delta = 1.0/((double) n);

  if (delta > 0.25)
    delta = 0.25;

  n = (1/delta);
  
  impulse_geom_bezier_delta(curve, delta, &d);

  cx = d.x;
  cy = d.y;

  s = *npoints;
  e = *npoints+n;
  
  for (x=s; x<e; x++)
    {
      *npoints += 1;
      *points = impulse_realloc(*points, sizeof(XPoint)* *npoints);

      (*points)[x].x = cx;
      (*points)[x].y = cy;

      cx += d.Dx;
      cy += d.Dy;

      d.Dx += d.D2x;
      d.Dy += d.D2y;

      d.D2x += d.D3x;
      d.D2y += d.D3y;
    }

  *npoints += 1;
  *points = impulse_realloc(*points, sizeof(XPoint)* *npoints);

  (*points)[x].x = curve->p[3].x;
  (*points)[x].y = curve->p[3].y;
}

void ROWS_screen_brush_draw_shape(ROWS_drawable_fparam *param)
{
  _p_ROWS_brush_data      **data;
  _p_ROWS_X_screen_data **drawable;
  _p_ROWS_X_brush_data     *Xdata;
  
  ROWS_brush_shape         *shape;
  int                       x, n;

  XPoint                   *points;
  int                       npoints;

  impulse_geom_bezier       curve;

  data     = (_p_ROWS_brush_data **)    param->brush.brush;
  drawable = (_p_ROWS_X_screen_data **) ROWS_screen_cast((*data)->drawable);
  Xdata    = (*data)->data;

  shape = param->brush.data.shape.shape;
  n = param->brush.data.shape.nopoints;
  if (n<0)
    {
      return;
    }

  impulse_mutex_X_lock;
  _p_ROWS_screen_brush_set_gc(param->brush.brush);
  
  points = impulse_alloc(sizeof(XPoint));
  points[0].x = MICRONS_TO_PIXEL(shape[0].x);
  points[0].y = MICRONS_TO_PIXEL(shape[0].y);
  npoints = 1;
  
  for (x=1; x<n; x++)
    {
      switch (shape[x].type)
	{
	case ROWS_BRUSH_LINE:
	  points = impulse_realloc(points,
				   sizeof(XPoint)*(npoints+1));
	  points[npoints].x = MICRONS_TO_PIXEL(shape[x].x);
	  points[npoints].y = MICRONS_TO_PIXEL(shape[x].y);
	  break;

	case ROWS_BRUSH_BEZIER:
	  curve.p[0].x = MICRONS_TO_PIXEL(shape[x-1].x);
	  curve.p[0].y = MICRONS_TO_PIXEL(shape[x-1].y);
	  
	  curve.p[1].x = MICRONS_TO_PIXEL(shape[x].x1);
	  curve.p[1].y = MICRONS_TO_PIXEL(shape[x].y1);
	  
	  curve.p[2].x = MICRONS_TO_PIXEL(shape[x].x2);
	  curve.p[2].y = MICRONS_TO_PIXEL(shape[x].y2);

	  curve.p[3].x = MICRONS_TO_PIXEL(shape[x].x);
	  curve.p[3].y = MICRONS_TO_PIXEL(shape[x].y);

	  make_bezier(&curve, &points, &npoints);
	  break;
	  
	case ROWS_BRUSH_MOVE:
	  /* FIXME */
	  break;
	}
    }

  XDrawLines((*drawable)->display,
	     (*drawable)->drawable,
	     Xdata->private_gc,
	     points,
	     npoints,
	     CoordModeOrigin);
  impulse_free(points);
  
  impulse_mutex_X_unlock;  
}

/* End of file */
