/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * style.c
 * -------
 *
 * Standard style functions
 */

#include "../../../config.h"

#include "impulse/rows/style.h"
#include "impulse/util/memory.h"
#include "impulse/util/error.h"
#include "impulse/util/measure.h"

ROWS_style ROWS_style_create(ROWS_drawable drawable)
{
  ROWS_colour background_grey;
  ROWS_colour highlight_grey;
  ROWS_colour inactive_grey;
  ROWS_colour white;
  ROWS_colour black;
  ROWS_colour shadow;

  ROWS_style style;
  int x;

  /* Standard colours */
  background_grey = ROWS_colour_rgb_for_drawable(drawable,3584, 3584, 3584);
  highlight_grey  = ROWS_colour_rgb_for_drawable(drawable,3840, 3840, 3840);
  inactive_grey   = ROWS_colour_rgb_for_drawable(drawable,1536, 1536, 1536);
  white           = ROWS_colour_rgb_for_drawable(drawable,4095, 4095, 4095);
  black           = ROWS_colour_rgb_for_drawable(drawable,0,    0,    0);
  shadow          = ROWS_colour_rgb_for_drawable(drawable,2048, 2048, 2048);

  style          = impulse_alloc(sizeof(struct _ROWS_style));
  style->brushes = impulse_alloc(sizeof(struct _ROWS_style_brush)*6);

  /* Turn the colours into brushes */
  style->brushes[0].fg_colour = background_grey;
  style->brushes[1].fg_colour = highlight_grey;
  style->brushes[2].fg_colour = inactive_grey;
  style->brushes[3].fg_colour = white;
  style->brushes[4].fg_colour = black;
  style->brushes[5].fg_colour = shadow;

  style->brushes[0].bg_colour = background_grey;
  style->brushes[1].bg_colour = highlight_grey;
  style->brushes[2].bg_colour = inactive_grey;
  style->brushes[3].bg_colour = background_grey;
  style->brushes[4].bg_colour = background_grey;
  style->brushes[5].bg_colour = background_grey;

  style->nbrushes = 6;

  for (x=0;x<style->nbrushes;x++)
    {
      style->brushes[x].brush = ROWS_brush_cast(ROWS_brush_create(drawable));
      ROWS_brush_set_fgcolour(style->brushes[x].brush,
			      style->brushes[x].fg_colour);
      ROWS_brush_set_bgcolour(style->brushes[x].brush,
			      style->brushes[x].bg_colour);
    }

  /* Fixme: TEST! */
  {
    ROWS_brush_dotpattern dp;
    ROWS_brush_dotpattern dp2;

    dp = malloc(sizeof(struct _ROWS_brush_dotpattern));
    dp2 = malloc(sizeof(struct _ROWS_brush_dotpattern));
    
    dp->next = dp2;
    dp2->next = NULL;
    
    dp->solid_len = PIXEL_TO_MICRONS(4);
    dp->empty_len = PIXEL_TO_MICRONS(2);

    dp2->solid_len = PIXEL_TO_MICRONS(2);
    dp2->empty_len = PIXEL_TO_MICRONS(2);

    ROWS_brush_set_dotpattern(style->brushes[4].brush,
			      dp,
			      0);

    free(dp);
    free(dp2);
  }

  /* Set up fonts */
  
  for (x=0;x<style->nbrushes;x++)
    {
      ROWS_brush_set_font(style->brushes[x].brush,
			  "fixed");
    }

  /* Set up the style block */
  style->brush[ROWS_STYLE_NORMAL][ROWS_STYLE_BACKGROUND]    =
    style->brushes[0].brush;
  style->brush[ROWS_STYLE_NORMAL][ROWS_STYLE_FOREGROUND]    =
    style->brushes[4].brush;
  style->brush[ROWS_STYLE_NORMAL][ROWS_STYLE_WRITABLE_BACK] =
    style->brushes[3].brush;
  style->brush[ROWS_STYLE_NORMAL][ROWS_STYLE_BORDERSHADOW]  =
    style->brushes[5].brush;
  style->brush[ROWS_STYLE_NORMAL][ROWS_STYLE_BORDERHILITE]  =
    style->brushes[3].brush;

  style->brush[ROWS_STYLE_INACTIVE][ROWS_STYLE_BACKGROUND]    =
    style->brushes[0].brush;
  style->brush[ROWS_STYLE_INACTIVE][ROWS_STYLE_FOREGROUND]    =
    style->brushes[2].brush;
  style->brush[ROWS_STYLE_INACTIVE][ROWS_STYLE_WRITABLE_BACK] =
    style->brushes[3].brush;
  style->brush[ROWS_STYLE_INACTIVE][ROWS_STYLE_BORDERSHADOW]  =
    style->brushes[5].brush;
  style->brush[ROWS_STYLE_INACTIVE][ROWS_STYLE_BORDERHILITE]  =
    style->brushes[3].brush;

  style->brush[ROWS_STYLE_HIGHLIGHT][ROWS_STYLE_BACKGROUND]    =
    style->brushes[1].brush;
  style->brush[ROWS_STYLE_HIGHLIGHT][ROWS_STYLE_FOREGROUND]    =
    style->brushes[4].brush;
  style->brush[ROWS_STYLE_HIGHLIGHT][ROWS_STYLE_WRITABLE_BACK] =
    style->brushes[3].brush;
  style->brush[ROWS_STYLE_HIGHLIGHT][ROWS_STYLE_BORDERSHADOW]  =
    style->brushes[5].brush;
  style->brush[ROWS_STYLE_HIGHLIGHT][ROWS_STYLE_BORDERHILITE]  =
    style->brushes[3].brush;

  style->brush[ROWS_STYLE_FOCUS][ROWS_STYLE_BACKGROUND]    =
    style->brushes[1].brush;
  style->brush[ROWS_STYLE_FOCUS][ROWS_STYLE_FOREGROUND]    =
    style->brushes[4].brush;
  style->brush[ROWS_STYLE_FOCUS][ROWS_STYLE_WRITABLE_BACK] =
    style->brushes[3].brush;
  style->brush[ROWS_STYLE_FOCUS][ROWS_STYLE_BORDERSHADOW]  =
    style->brushes[5].brush;
  style->brush[ROWS_STYLE_FOCUS][ROWS_STYLE_BORDERHILITE]  =
    style->brushes[3].brush;

  style->brush[ROWS_STYLE_ACTIVE][ROWS_STYLE_BACKGROUND]    =
    style->brushes[1].brush;
  style->brush[ROWS_STYLE_ACTIVE][ROWS_STYLE_FOREGROUND]    =
    style->brushes[4].brush;
  style->brush[ROWS_STYLE_ACTIVE][ROWS_STYLE_WRITABLE_BACK] =
    style->brushes[3].brush;
  style->brush[ROWS_STYLE_ACTIVE][ROWS_STYLE_BORDERSHADOW]  =
    style->brushes[5].brush;
  style->brush[ROWS_STYLE_ACTIVE][ROWS_STYLE_BORDERHILITE]  =
    style->brushes[3].brush;

  style->usage = 0;

  return style;
}

void ROWS_style_reference(ROWS_style style)
{
  if (style != NULL)
    style->usage++;
}

void ROWS_style_dereference(ROWS_style style)
{
  if (style != NULL)
    {
      style->usage--;
      if (style->usage<=0)
	ROWS_style_destroy(style);
    }
}

void ROWS_style_destroy(ROWS_style style)
{
  int x;

  if (style->usage > 0)
  {
    impulse_unistring desc = impulse_unistring_create_utf("Style destroyed while still in use", 8);
    impulse_error_raise(IMPULSE_ERROR_WARNING, desc, NULL);
    impulse_unistring_destroy(desc);
  }
  
  for (x=0; x<style->nbrushes; x++)
    {
      ROWS_widget_destroy((void **) style->brush[x]);
    }

  impulse_free(style);
}


