/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Danie Berrange
 * are Copyright (C) 1999 Daniel Berrange
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * error.c
 * -------
 *
 * Impulse program error handling
 */

#ifndef __impulse_util_error_h
#define __impulse_util_error_h

#include <impulse/util/unistring.h>

#ifdef __cplusplus
extern "C" {  
#endif

  /*                            ======--------======                           */

  /* 
   * The different types of errors you can raise
   */

  typedef enum
  {
    IMPULSE_ERROR_EMERGENCY = 0, /* Get the hell out quickly, no cleanup stuff */
    IMPULSE_ERROR_CRITICAL,      /* Shutdown gracefully, calling cleanup funcs */
    IMPULSE_ERROR_WARNING,       /* Its starting to look bad, but carry on anyway */
    IMPULSE_ERROR_INFO,          /* Just so they know whats going on */
    
    IMPULSE_ERROR_MAX = IMPULSE_ERROR_INFO
  } impulse_error_level;
  

  /*
   * Allow the calling of a program specific error handler
   */
  typedef void (*impulse_error_handler)(impulse_error_level,
					impulse_unistring,
					...);
  
  /*
   * Actually trigger the error
   */
  void impulse_error_raise(impulse_error_level level,
			   impulse_unistring format,
			   ...);
  
  /*
   * Decide what happens according to error severity
   */
  void impulse_error_handle(impulse_error_level level,
			    impulse_error_handler handler);
  
  
  void impulse_error_syscall(impulse_error_level level,
			     impulse_unistring format,
			     ...);
  
  
  /* 
   * Some builtin error handlers to play with
   */
  
  void impulse_error_print(impulse_error_level level,
			   impulse_unistring format,
			   ...);
  void impulse_error_log(impulse_error_level level,
			 impulse_unistring format,
			 ...);
  void impulse_error_bug(impulse_error_level level,
			 impulse_unistring format,
			 ...);

#ifdef __cplusplus
}
#endif

#endif

/* End of file */
