/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Daniel Berrange. Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * hash.h
 * ------
 *
 * Hash tables
 */

#ifndef __impulse_util_hash_h
#define __impulse_util_hash_h

#include <impulse/util/unistring.h>
#include <impulse/util/iterator.h>

#ifdef __cplusplus
extern "C" {  
#endif

/*                            ======--------======                           */

/* Hashing data structures */

typedef struct _impulse_hash *impulse_hash;
typedef void (*impulse_hash_cleaner_func)(void *data);

/*                            ======--------======                           */

/* Hashing functions */

impulse_hash impulse_hash_create(void);
void impulse_hash_destroy(impulse_hash table);


void impulse_hash_load(impulse_hash table,
		       impulse_float factor);
void impulse_hash_pack(impulse_hash table);
void impulse_hash_cleaner(impulse_hash hash,
			  impulse_hash_cleaner_func func);
impulse_iterator impulse_hash_elements(impulse_hash hash);
impulse_iterator impulse_hash_keys(impulse_hash hash);

impulse_bool impulse_hash_empty(impulse_hash hash);
impulse_uint impulse_hash_count(impulse_hash hash);
impulse_uint impulse_hash_capacity(impulse_hash hash);

void impulse_hash_store(impulse_hash table,
			impulse_unistring key,
			void *data);
void *impulse_hash_recover(impulse_hash table,
			   impulse_unistring key);

void *impulse_hash_remove(impulse_hash hash,
			 impulse_unistring key);
void impulse_hash_clear(impulse_hash table);
impulse_bool impulse_hash_contains(impulse_hash table,
				   void *data);
impulse_bool impulse_hash_contains_key(impulse_hash table,
				       impulse_unistring key);

#ifdef __cplusplus
}
#endif

#endif

