/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * main.h
 * ------
 *
 * Impulse program startup and termination functions
 */


#ifndef __impulse_util_app_h
#define __impulse_util_app_h
    
#include <impulse/util/unistring.h>
#include <impulse/util/text.h>

#ifdef __cplusplus
extern "C" {  
#endif

  typedef enum _impulse_app_arg_type
  {
    IMPULSE_APP_ARG_NONE,
    IMPULSE_APP_ARG_OPTIONAL,
    IMPULSE_APP_ARG_SINGLE,
    IMPULSE_APP_ARG_MULTIPLE
  } impulse_app_arg_type;
  
  typedef enum _impulse_app_dir_type
  {
    IMPULSE_APP_DIR_HOME,
    IMPULSE_APP_DIR_TEMP,
      
    IMPULSE_APP_DIR_MAX
  } impulse_app_dir_type;
  
  typedef enum _impulse_app_name_type
  {
    IMPULSE_APP_NAME_PROGRAM,
    IMPULSE_APP_NAME_USER,
    IMPULSE_APP_NAME_GROUP,
    IMPULSE_APP_NAME_HOST,
    IMPULSE_APP_NAME_PERSON,
      
    IMPULSE_APP_NAME_MAX
  } impulse_app_name_type;

  typedef void (*impulse_command_func)(void);
  
  /*                            ======--------======                           */

  void impulse_init(impulse_uint argc,
		    impulse_char **argv,
		    impulse_char *progname);
  void impulse_cleanup(void);
  
  /* Flow control */
  
  void impulse_main(void);
  void impulse_quit(void);
  impulse_bool impulse_is_quit(void);

  /* Command line args */
  
  void impulse_command_parse(void);
  void impulse_command_option(impulse_unistring lopt,
			      impulse_unichar sopt,
			      impulse_unistring help,
			      impulse_app_arg_type type);
  impulse_uint impulse_command_argument(impulse_unistring lopt,
					impulse_unistring **values);
  void impulse_command_hook(impulse_command_func func);
  void impulse_command_help(void);
  
  /* Property query */

  impulse_unistring impulse_app_dir(impulse_app_dir_type type);
  impulse_unistring impulse_app_name(impulse_app_name_type type);
  impulse_unistring impulse_app_command(void);
  
  /* Internationalisation */

  impulse_unistring impulse_app_locale(void);
  impulse_unistring impulse_app_charset(void);
  
#ifdef __cplusplus
}
#endif
      
#endif __impulse_app_h

/* End of file */
