/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the Impulse utility library
 * for Impulse. (Part of the ROWS distribution)
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * Portions created by Daniel Berrange
 * are Copyright (C) 1999 Daniel Berrange.
 */

/*
 * memory.c
 * --------
 *
 * Memory handling functions
 */

#ifndef __impulse_util_memory_h
#define __impulse_util_memory_h

#include <stddef.h>
#include <impulse/util/setup.h>

#ifdef __cplusplus
extern "C" {  
#endif
#define CHECK_MEMORY_LEAKS 1

  /*
   * If you get mysterious segfaults while using Impulse, it's
   * possible that your machine needs weird alignment
   */
#define IMPULSE_MEMORY_PADDING 0

  typedef struct _impulse_memory_allocator
  {
    int line;
    char *source_file;
    char *comment;
    size_t size;

    struct _impulse_mem_list *item;
  } impulse_memory_allocator;
  
  typedef struct _impulse_memory_meta
  {
    impulse_memory_allocator *allocate;
    void                     *user;
    impulse_uint              count;
  } impulse_memory_meta;
  
  typedef struct _impulse_memory_block
  {
#if CHECK_MEMORY_LEAKS
    impulse_uint         magic; /* (maybe truncated) memory address */
#endif
    impulse_memory_meta *meta;  /* Meta data (where applicable) */
#if IMPULSE_MEMORY_PADDING>0
    char padding[PADDING];
#endif

    /* The allocated memory itself goes here */
  } impulse_memory_block;

  void impulse_mcheck   (void);
  void impulse_mem_used (void *mem);
  int  impulse_mem_count(void *mem);
  
#if !CHECK_MEMORY_LEAKS
  void *impulse_alloc(size_t size);
  void *impulse_realloc(void *old, size_t size);
  void impulse_free(void *old);

#define impulse_alloc_str(size, comment) impulse_alloc(size)
#else
  void *impulse_alloc_c(size_t size,
			int line_no,
			char *const file,
			char *const comment);
  void *impulse_realloc_c(void   *old,
			  size_t  size,
			  int line_no,
			  char *const file);
  void  impulse_free(void *old);

  struct _impulse_mem_list
  {
    impulse_memory_block *d;
    
    struct _impulse_mem_list *next;
    struct _impulse_mem_list *last;
  };

  struct _impulse_mem_stats
  {
    size_t current;
    size_t maximum;
    size_t total;
    size_t total_freed;

    impulse_uint num_blocks;
    impulse_uint max_blocks;
  };

  struct _impulse_mem_stats  impulse_memory_get_stats(void);
  struct _impulse_mem_list  *impulse_memory_get_allocated(void);

#define impulse_realloc(buf, size) impulse_realloc_c(buf, size, __LINE__, __FILE__)
#define impulse_alloc(size) impulse_alloc_c(size, __LINE__, __FILE__, NULL)
#define impulse_alloc_str(size, comment) impulse_alloc_c(size, __LINE__, __FILE__, comment)
#endif

#ifdef __cplusplus
}
#endif

#endif

/* End of file */
