/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * unichar.h
 * ---------
 *
 * Functions to manipulate Unicode characters
 */

#include "../../../config.h"

#include "impulse/util/unichar.h"
#include "impulse/util/unidata.h"

/*                            ======--------======                           */
impulse_uint impulse_unichar_category(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][(c&0xff)].category;
}


impulse_uint impulse_unichar_direction(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][(c&0xff)].direction;
}


impulse_uint impulse_unichar_combining(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][(c&0xff)].combining;
}


impulse_bool impulse_unichar_mirrored(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][(c&0xff)].mirrored;
}


impulse_uint impulse_unichar_formatting(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][(c&0xff)].formatting;
}


impulse_unichar *impulse_unichar_decompose(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][(c&0xff)].decomposition;
}


impulse_unichar impulse_unichar_compose(impulse_unichar *c)
{
  /*
   * Fix me: implement me!
   */
  return 0;
}


/* 
 * Some convenience functions for commonly used category info
 */

impulse_bool impulse_unichar_isalnum(impulse_unichar c)
{
  impulse_uint category = impulse_unidata[(c>>8)&0xff][c&0xff].category;
  return category >= IMPULSE_UNICHAR_LETTER_UPPERCASE &&
    category <= IMPULSE_UNICHAR_NUMBER_OTHER;
}


impulse_bool impulse_unichar_isalpha(impulse_unichar c)
{
  impulse_uint category = impulse_unidata[(c>>8)&0xff][c&0xff].category;
  return category >= IMPULSE_UNICHAR_LETTER_UPPERCASE &&
    category <= IMPULSE_UNICHAR_LETTER_OTHER;
}


impulse_bool impulse_unichar_isnumber(impulse_unichar c)
{
  impulse_uint category = impulse_unidata[(c>>8)&0xff][c&0xff].category;
  return category >= IMPULSE_UNICHAR_NUMBER_DECIMAL &&
    category <= IMPULSE_UNICHAR_NUMBER_OTHER;
}


impulse_bool impulse_unichar_isdigit(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].category == IMPULSE_UNICHAR_NUMBER_DECIMAL;
}


impulse_bool impulse_unichar_isascii(impulse_unichar c)
{
  return c <= IMPULSE_BYTE_MAX;
}


impulse_bool impulse_unichar_isgraph(impulse_unichar c)
{
  impulse_uint category = impulse_unidata[(c>>8)&0xff][c&0xff].category;
  return category >= IMPULSE_UNICHAR_LETTER_UPPERCASE &&
    category <= IMPULSE_UNICHAR_MARK_ENCLOSING;
}


impulse_bool impulse_unichar_isprint(impulse_unichar c)
{
  impulse_uint category = impulse_unidata[(c>>8)&0xff][c&0xff].category;
  return category >= IMPULSE_UNICHAR_LETTER_UPPERCASE &&
    category <= IMPULSE_UNICHAR_SEP_PARAGRAPH;
}


impulse_bool impulse_unichar_ispunct(impulse_unichar c)
{
  impulse_uint category = impulse_unidata[(c>>8)&0xff][c&0xff].category;
  return category >= IMPULSE_UNICHAR_PUNCT_CONNECTOR &&
    category <= IMPULSE_UNICHAR_PUNCT_OTHER;
}


impulse_bool impulse_unichar_isspace(impulse_unichar c)
{
  impulse_uint category = impulse_unidata[(c>>8)&0xff][c&0xff].category;
  return category >= IMPULSE_UNICHAR_SEP_SPACE &&
    category <= IMPULSE_UNICHAR_SEP_PARAGRAPH;
}


impulse_bool impulse_unichar_issymbol(impulse_unichar c)
{
  impulse_uint category = impulse_unidata[(c>>8)&0xff][c&0xff].category;
  return category >= IMPULSE_UNICHAR_SYMBOL_MATH &&
    category <= IMPULSE_UNICHAR_SYMBOL_OTHER;
}


impulse_bool impulse_unichar_ismark(impulse_unichar c)
{
  impulse_uint category = impulse_unidata[(c>>8)&0xff][c&0xff].category;
  return category >= IMPULSE_UNICHAR_MARK_NON_SPACING &&
    category <= IMPULSE_UNICHAR_MARK_ENCLOSING;
}


impulse_bool impulse_unichar_isother(impulse_unichar c)
{
  impulse_uint category = impulse_unidata[(c>>8)&0xff][c&0xff].category;
  return category >= IMPULSE_UNICHAR_OTHER_CONTROL &&
    category <= IMPULSE_UNICHAR_OTHER_NOT_ASSIGNED;
}


impulse_bool impulse_unichar_islower(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].category == IMPULSE_UNICHAR_LETTER_LOWERCASE;
}


impulse_bool impulse_unichar_isupper(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].category == IMPULSE_UNICHAR_LETTER_UPPERCASE;
}


impulse_bool impulse_unichar_istitle(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].category == IMPULSE_UNICHAR_LETTER_TITLECASE;
}


impulse_bool impulse_unichar_iscontrol(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].category == IMPULSE_UNICHAR_OTHER_CONTROL;
}


impulse_bool impulse_unichar_isformat(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].category == IMPULSE_UNICHAR_OTHER_FORMAT;
}


impulse_bool impulse_unichar_issurrogate(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].category == IMPULSE_UNICHAR_OTHER_SURROGATE;
}


impulse_bool impulse_unichar_isprivate(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].category == IMPULSE_UNICHAR_OTHER_PRIVATE_USE;
}


impulse_bool impulse_unichar_isassigned(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].category != IMPULSE_UNICHAR_OTHER_NOT_ASSIGNED;
}


/* 
 * Some convenience functions for directional information
 */

impulse_bool impulse_unichar_islr(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].direction == IMPULSE_UNICHAR_LEFT_RIGHT;
}


impulse_bool impulse_unichar_isrl(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].direction == IMPULSE_UNICHAR_RIGHT_LEFT;
}


impulse_bool impulse_unichar_isneutral(impulse_unichar c)
{
  impulse_uint direction = impulse_unidata[(c>>8)&0xff][c&0xff].direction;
  return direction >= IMPULSE_UNICHAR_NEUTRAL_WHITESPACE &&
    direction <= IMPULSE_UNICHAR_NEUTRAL_OTHER;
}


impulse_bool impulse_unichar_isseparator(impulse_unichar c)
{
  impulse_uint direction = impulse_unidata[(c>>8)&0xff][c&0xff].direction;
  return direction >= IMPULSE_UNICHAR_BLOCK_SEPARATOR &&
    direction <= IMPULSE_UNICHAR_SEGMENT_SEPARATOR;
}


/* 
 * Case conversion functions 
 */

impulse_unichar impulse_unichar_totitle(impulse_unichar c)
{
  impulse_unidata_item block = impulse_unidata[(c>>8)&0xff][c&0xff];
  return block.titlecase ? block.titlecase : c;
}


impulse_unichar impulse_unichar_tolower(impulse_unichar c)
{
  impulse_unidata_item block = impulse_unidata[(c>>8)&0xff][c&0xff];
  return block.lowercase ? block.lowercase : c;
}


impulse_unichar impulse_unichar_toupper(impulse_unichar c)
{
  impulse_unidata_item block = impulse_unidata[(c>>8)&0xff][c&0xff];
  return block.uppercase ? block.uppercase : c;
}


impulse_float impulse_unichar_value(impulse_unichar c)
{
  return impulse_unidata[(c>>8)&0xff][c&0xff].value;
}


/* End of file */
