/*                                                                  |- CFSFC -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is Copyright (C) 1998 Andrew Hunter.
 * All Rights Reserved. 
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter.
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * unichar.h
 * ---------
 *
 * Data structures to represent, and functions to manipulate, Unicode characters.
 */

#ifndef __impulse_util_unichar_h
#define __impulse_util_unichar_h
    
#include <impulse/util/setup.h>

#ifdef __cplusplus
extern "C" {  
#endif

  /*
   * General category flags
   * 
   * These are numbered in an order designed to simply the ctype equiv functions.
   * Don't go changing it without thinking very carefully about the consequences.
   * 
   *  -- DPB  23/2/00
   */
#define IMPULSE_UNICHAR_LETTER_UPPERCASE 1 /* Lu */
#define IMPULSE_UNICHAR_LETTER_LOWERCASE 2 /* Ll */
#define IMPULSE_UNICHAR_LETTER_TITLECASE 3 /* Lt */
#define IMPULSE_UNICHAR_LETTER_MODIFIER  4 /* Lm */
#define IMPULSE_UNICHAR_LETTER_OTHER     5 /* Lo */

#define IMPULSE_UNICHAR_NUMBER_DECIMAL   6 /* Nd */
#define IMPULSE_UNICHAR_NUMBER_LETTER    7 /* Nl */
#define IMPULSE_UNICHAR_NUMBER_OTHER     8 /* No */

#define IMPULSE_UNICHAR_PUNCT_CONNECTOR     9 /* Pc */
#define IMPULSE_UNICHAR_PUNCT_DASH          10 /* Pd */
#define IMPULSE_UNICHAR_PUNCT_OPEN          11 /* Ps */
#define IMPULSE_UNICHAR_PUNCT_CLOSE         12 /* Pe */
#define IMPULSE_UNICHAR_PUNCT_INITIAL_QUOTE 13 /* Pi */
#define IMPULSE_UNICHAR_PUNCT_FINAL_QUOTE   14 /* Pf */
#define IMPULSE_UNICHAR_PUNCT_OTHER         15 /* Po */
  
#define IMPULSE_UNICHAR_SYMBOL_MATH     16 /* Sm */
#define IMPULSE_UNICHAR_SYMBOL_CURRENCY 17 /* Sc */
#define IMPULSE_UNICHAR_SYMBOL_MODIFIER 18 /* Sk */
#define IMPULSE_UNICHAR_SYMBOL_OTHER    19 /* So */

#define IMPULSE_UNICHAR_MARK_NON_SPACING 20 /* Mn */
#define IMPULSE_UNICHAR_MARK_SPACING     21 /* Mc */
#define IMPULSE_UNICHAR_MARK_ENCLOSING   22 /* Me */
  
#define IMPULSE_UNICHAR_SEP_SPACE        23 /* Zs */
#define IMPULSE_UNICHAR_SEP_LINE         24 /* Zl */
#define IMPULSE_UNICHAR_SEP_PARAGRAPH    25 /* Zp */

#define IMPULSE_UNICHAR_OTHER_CONTROL      26 /* Cc */
#define IMPULSE_UNICHAR_OTHER_FORMAT       27 /* Cf */
#define IMPULSE_UNICHAR_OTHER_SURROGATE    28 /* Cs */
#define IMPULSE_UNICHAR_OTHER_PRIVATE_USE  29 /* Co */
#define IMPULSE_UNICHAR_OTHER_NOT_ASSIGNED 30 /* Cn */
  
  
  /*
   * Directional class info
   */
#define IMPULSE_UNICHAR_LEFT_RIGHT              1  /* L, LRE, LRO */
#define IMPULSE_UNICHAR_RIGHT_LEFT              2  /* R, AL, RLE, RLO */
  
  /* Weak types */
#define IMPULSE_UNICHAR_EURO_NUMBER             3 /* EN */
#define IMPULSE_UNICHAR_EURO_NUMBER_SEPARATOR   4 /* ES */
#define IMPULSE_UNICHAR_EURO_NUMBER_TERMINATOR  5 /* ET */
#define IMPULSE_UNICHAR_ARABIC_NUMBER           6 /* AN */
#define IMPULSE_UNICHAR_COMMON_NUMBER_SEPARATOR 7 /* CS */
  
  /* Separators */
#define IMPULSE_UNICHAR_BLOCK_SEPARATOR         8 /* B */
#define IMPULSE_UNICHAR_SEGMENT_SEPARATOR       9 /* S */
  
  /* Neutrals */
#define IMPULSE_UNICHAR_NEUTRAL_WHITESPACE      10 /* WS */
#define IMPULSE_UNICHAR_NEUTRAL_OTHER           11 /* ON */
  

  /*
   * Combining class info
   */
#define IMPULSE_UNICHAR_COMBINING_SPACING  0
#define IMPULSE_UNICHAR_COMBINING_OVERLAY  1
#define IMPULSE_UNICHAR_COMBINING_TIBETAN_SUBJOINED  6
#define IMPULSE_UNICHAR_COMBINING_NUKTAS  7
#define IMPULSE_UNICHAR_COMBINING_HIRAGANA  8
#define IMPULSE_UNICHAR_COMBINING_KATAKANA  8
#define IMPULSE_UNICHAR_COMBINING_VIRAMAS  9
#define IMPULSE_UNICHAR_COMBINING_START_FIXED  10
#define IMPULSE_UNICHAR_COMBINING_END_FIXED  199
#define IMPULSE_UNICHAR_COMBINING_BELOW_LEFT_ATTACHED  200
#define IMPULSE_UNICHAR_COMBINING_BELOW_ATTACHED  202
#define IMPULSE_UNICHAR_COMBINING_BELOW_RIGHT_ATTACHED  204
#define IMPULSE_UNICHAR_COMBINING_LEFT_ATTACHED  208
#define IMPULSE_UNICHAR_COMBINING_RIGHT_ATTACHED  210
#define IMPULSE_UNICHAR_COMBINING_ABOVE_LEFT_ATTACHED  212
#define IMPULSE_UNICHAR_COMBINING_ABOVE_ATTACHED  214
#define IMPULSE_UNICHAR_COMBINING_ABOVE_RIGHT_ATTACHED  216
#define IMPULSE_UNICHAR_COMBINING_BELOW_LEFT  218
#define IMPULSE_UNICHAR_COMBINING_BELOW  220
#define IMPULSE_UNICHAR_COMBINING_BELOW_RIGHT  222
#define IMPULSE_UNICHAR_COMBINING_LEFT  224
#define IMPULSE_UNICHAR_COMBINING_RIGHT  226
#define IMPULSE_UNICHAR_COMBINING_ABOVE_LEFT  228
#define IMPULSE_UNICHAR_COMBINING_ABOVE  230
#define IMPULSE_UNICHAR_COMBINING_ABOVE_RIGHT  232
#define IMPULSE_UNICHAR_COMBINING_DOUBLE_ABOVE  234

  /*
   * Formatting info
   */
#define IMPULSE_UNICHAR_FORMAT_FONT     1 /* font */
#define IMPULSE_UNICHAR_FORMAT_NOBREAK  2 /* noBreak */
#define IMPULSE_UNICHAR_FORMAT_INITIAL  3 /* initial */
#define IMPULSE_UNICHAR_FORMAT_MEDIAL   4 /* medial */
#define IMPULSE_UNICHAR_FORMAT_FINAL    5 /* final */
#define IMPULSE_UNICHAR_FORMAT_ISOLATED 6 /* isolated */
#define IMPULSE_UNICHAR_FORMAT_CIRCLE   7 /* circle */
#define IMPULSE_UNICHAR_FORMAT_SUPER    8 /* super */
#define IMPULSE_UNICHAR_FORMAT_SUB      9 /* sub */
#define IMPULSE_UNICHAR_FORMAT_VERTICAL 10 /* vertical */
#define IMPULSE_UNICHAR_FORMAT_WIDE     11 /* wide */
#define IMPULSE_UNICHAR_FORMAT_NARROW   12 /* narrow */
#define IMPULSE_UNICHAR_FORMAT_SMALL    13 /* small */
#define IMPULSE_UNICHAR_FORMAT_SQUARE   14 /* square */
#define IMPULSE_UNICHAR_FORMAT_FRACTION 15 /* fraction */
#define IMPULSE_UNICHAR_FORMAT_COMPAT   16 /* compat */


  /* Assorted interesting character ranges */

#define IMPULSE_UNICHAR_SURR_HIGH_START (0xD800)
#define IMPULSE_UNICHAR_SURR_HIGH_END   (0xDB7F)
#define IMPULSE_UNICHAR_SURR_LOW_START  (0xDC00)
#define IMPULSE_UNICHAR_SURR_LOW_END    (0xDFFF)

  
  /*                            ======--------======                           */
  
  impulse_uint impulse_unichar_category(impulse_unichar c);
  impulse_uint impulse_unichar_direction(impulse_unichar c);
  impulse_uint impulse_unichar_combining(impulse_unichar c);
  impulse_bool impulse_unichar_mirrored(impulse_unichar c);
  impulse_uint impulse_unichar_formatting(impulse_unichar c);
  impulse_unichar *impulse_unichar_decompose(impulse_unichar c);
  impulse_unichar impulse_unichar_compose(impulse_unichar *c);
  
  /* 
   * Some convenience functions for commonly used category info
   */
  
  impulse_bool impulse_unichar_isalnum(impulse_unichar c);
  impulse_bool impulse_unichar_isalpha(impulse_unichar c);
  impulse_bool impulse_unichar_isnumber(impulse_unichar c);
  impulse_bool impulse_unichar_isdigit(impulse_unichar c);
  impulse_bool impulse_unichar_isascii(impulse_unichar c);
  
  impulse_bool impulse_unichar_isgraph(impulse_unichar c);
  impulse_bool impulse_unichar_isprint(impulse_unichar c);
  impulse_bool impulse_unichar_ispunct(impulse_unichar c);
  impulse_bool impulse_unichar_isspace(impulse_unichar c);
  impulse_bool impulse_unichar_issymbol(impulse_unichar c);
  impulse_bool impulse_unichar_ismark(impulse_unichar c);
  impulse_bool impulse_unichar_isother(impulse_unichar c);

  impulse_bool impulse_unichar_islower(impulse_unichar c);
  impulse_bool impulse_unichar_isupper(impulse_unichar c);
  impulse_bool impulse_unichar_istitle(impulse_unichar c);

  impulse_bool impulse_unichar_iscontrol(impulse_unichar c);
  impulse_bool impulse_unichar_isformat(impulse_unichar c);
  impulse_bool impulse_unichar_issurrogate(impulse_unichar c);
  impulse_bool impulse_unichar_isprivate(impulse_unichar c);
  impulse_bool impulse_unichar_isassigned(impulse_unichar c);


  /* 
   * Some convenience functions for directional information
   */
  
  impulse_bool impulse_unichar_islr(impulse_unichar c);
  impulse_bool impulse_unichar_isneutral(impulse_unichar c);
  impulse_bool impulse_unichar_isseparater(impulse_unichar c);
  
  /* 
   * Case conversion functions 
   */
  
  impulse_unichar impulse_unichar_totitle(impulse_unichar c);
  impulse_unichar impulse_unichar_tolower(impulse_unichar c);
  impulse_unichar impulse_unichar_toupper(impulse_unichar c);

/*                            ======--------======                           */

#ifdef __cplusplus
}
#endif

#endif
