/*                                                                |- Impulse -|
 * The contents of this file are subject to the Mozilla
 * Public License Version 1.0 (the "License"); you may
 * not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 *
 * Software distributed under the License is distributed
 * on an "AS IS" basis, WITHOUT WARRANTY OF
 * ANY KIND, either express or implied. See the
 * License for the specific language governing rights
 * and limitations under the License. 
 *
 * The Original Code is the ROWS widget library
 * for Impulse.
 *
 * The Initial Developer of the Original Code is
 * Andrew Hunter. Portions created by Andrew Hunter
 * are Copyright (C) 1999 Andrew Hunter.
 * All Rights Reserved
 *
 * Contributor(s):
 * No-one (yet :-)
 */

/*
 * ROWSbitmap_png.c
 * ----------------
 *
 * Routines to load PNG files (requires libpng)
 */

#include "config.h"

#include "modules/bitmap/_png.h"

#include "impulse/util/memory.h"
#include "impulse/rows/main.h"

#include <png.h>

impulse_int ROWS_bitmap_png_load(ROWS_bitmap bitmap,
				 impulse_unistring filename);
impulse_int ROWS_bitmap_png_save(ROWS_bitmap bitmap,
				 impulse_unistring filename);

struct _ROWS_bitmap_handler handler =
{
  ROWS_bitmap_png_load,
  ROWS_bitmap_png_save
};

ROWS_bitmap_handler png_handler = &handler;

#define HEADER_READ 4

impulse_int ROWS_bitmap_png_load(ROWS_bitmap bitmap,
				 impulse_unistring filename)
{
  FILE *png_file;
  impulse_char *cfile;
  impulse_char header[8];
  png_structp png_ptr;
  png_infop info_ptr, end_info;
  impulse_int x;
  
  png_uint_32 width, height;
  impulse_int bit_depth, colour_type;
printf("Loading png\n");  
  cfile = impulse_unistring_to_utf(filename, 8, 0);
  if (!(png_file = fopen(cfile, "rb")))
  {
    impulse_free(cfile);
    return 0;
  }
  impulse_free(cfile);
  
  /* Ok, check out the PNG file... */

  x = fread(header, 1, HEADER_READ, png_file);

  if (x < HEADER_READ)
    {
      printf("Not long enough");
      
      fclose(png_file);
      return 0;
    }
  
  if (png_sig_cmp(header, 0, HEADER_READ-1))
    {
      printf("Not a PNG\n");
      
      fclose(png_file);
      return 0; /* Not a PNG */
    }

  /* Right, now prepare for reading... */
  png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
				   NULL, /* FIXME */
				   NULL, /* FIXME */
				   NULL  /* FIXME */);
  if (!png_ptr)
    {
      printf("Erm\n");
      
      fclose(png_file);
      return 0;
    }

  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr)
    {
      printf("Oh dear\n");
      
      png_destroy_read_struct(&png_ptr,
			      (png_infopp)NULL, (png_infopp)NULL);
      return 0;
    }

  end_info = png_create_info_struct(png_ptr);
  if (!end_info)
    {
      printf("Darn");
      
      png_destroy_read_struct(&png_ptr, &info_ptr,
			      (png_infopp)NULL);
      return 0;
    }

  png_init_io(png_ptr, png_file);
  png_set_sig_bytes(png_ptr, HEADER_READ);

  png_read_info(png_ptr, info_ptr);
  png_get_IHDR (png_ptr,    info_ptr,
		&width,     &height,
		&bit_depth, &colour_type,
		NULL, NULL, NULL);

  printf("depth: %i width: %i height: %i\n", bit_depth, width, height);

  if (bit_depth == 16)
    png_set_strip_16(png_ptr); /*
				* Maximum depth of a ROWS bitmap is
				* 24bpp - 8 per channel
				*/

  /* FIXME: byte ordering for non x86 machines. libpng makes this easy... */
  if (bit_depth == 8 && colour_type == PNG_COLOR_TYPE_RGB)
    {
      png_set_filler(png_ptr, 0, PNG_FILLER_BEFORE);
    }

  if (colour_type == PNG_COLOR_TYPE_GRAY       ||
      colour_type == PNG_COLOR_TYPE_GRAY_ALPHA)
    png_set_gray_to_rgb(png_ptr);

  /* Finish up */
  png_read_end(png_ptr, NULL);

  printf("Read ended\n");

  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);

  printf("Memory freed\n");
  
  fclose(png_file);
  return 0;
}


impulse_int ROWS_bitmap_png_save(ROWS_bitmap bitmap,
				 impulse_unistring filename)
{
  printf("Done save\n");
  return 0;
}


/* End of file */
