dnl
dnl Core functions
dnl

dnl *****************************************************************
dnl * Initialise core configuration information
dnl *****************************************************************


AC_DEFUN(IMPULSE_CORE_INIT,
[
  DEBUG=0
  AC_ARG_ENABLE(debug, [  --enable-debug          Enable debugging output],
  [
    if test "$enableval" = yes; then
      DEBUG=1
    fi
  ])
])


dnl *****************************************************************
dnl * Check for core functionality
dnl *****************************************************************

AC_DEFUN(IMPULSE_CORE_CHECK,
[
  IMPULSE_MSG_SECTION([Checking core functionality])

  GLOBAL_CFLAGS="-D_POSIX_C_SOURCE=199506L -D_XOPEN_SOURCE=500"

  IMPULSE_CORE_CHECK_PROGRAMS
  IMPULSE_CORE_CHECK_HEADERS
  IMPULSE_CORE_CHECK_DEBUG

  CORE_CFLAGS="$CFLAGS $CORE_CFLAGS"
  CORE_LIBS="$LIBS $CORE_LIBS"
  CORE_LDFLAGS="$LDFLAGS $CORE_LDFLAGS"

  AC_CYGWIN
])


dnl *****************************************************************
dnl * Check for a few programs which we need
dnl *****************************************************************


AC_DEFUN(IMPULSE_CORE_CHECK_PROGRAMS,
[
  AC_PATH_PROG(PERL, perl)
  dnl These seem to be checked by libtool anyway
dnl  AC_PROG_INSTALL
dnl  AC_PROG_CC
  AC_PROG_CPP
dnl  AC_PROG_LN_S
dnl  AC_PROG_MAKE_SET
dnl  AC_PROG_RANLIB
  
  AC_PROG_YACC
  AM_PROG_LEX
])


dnl *****************************************************************
dnl * Check for a few more header files which we need
dnl *****************************************************************


AC_DEFUN(IMPULSE_CORE_CHECK_HEADERS,
[
  AC_HEADER_STDC
  AC_CHECK_HEADER(unistd.h,,AC_MSG_ERROR([ROWS needs a working unistd.h to compile]))
  AC_CHECK_HEADER(time.h,,AC_MSG_ERROR([ROWS needs a working time.h to compile]))
  AC_CHECK_HEADER(math.h,,AC_MSG_ERROR([ROWS needs a working math.h to compile]))

  AC_CHECK_LIB(m, pow)
])



dnl *****************************************************************
dnl * Check for a memory corruption checking version of malloc
dnl *****************************************************************


AC_DEFUN(IMPULSE_CORE_CHECK_DEBUG,
[
  IMPULSE_PARAMS_SAVE
  
  AC_CHECK_HEADER(mcheck.h,
  [
    AC_CACHE_CHECK([for mcheck], impulse_cv_mcheck,
    [
      AC_TRY_COMPILE([
        #include <mcheck.h>
        #include <stdlib.h>
      ],[
        mcheck(NULL);
      ],[
        AC_TRY_LINK([
          #include <mcheck.h>
          #include <stdlib.h>
        ],[
          mcheck(NULL);
        ],[
          impulse_cv_mcheck="yes"
        ],[
          impulse_cv_mcheck="no"
        ])
      ],[
        impulse_cv_mcheck="no"
      ])
    ])
    
    if test "$impulse_cv_mcheck" = "yes" ; then
      AC_DEFINE(HAVE_DEBUG_MCHECK, 1)
    fi
  ])
  
  AC_PATH_PROG(GDB, gdb, "none")
  if test "$GDB" != "none" ; then
    AC_DEFINE(HAVE_DEBUG_GDB, 1)
  fi

  
  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Finish up configuration
dnl *****************************************************************

AC_DEFUN(IMPULSE_CORE_FINISH,
[
  COMMON_CFLAGS="$COMMON_CFLAGS -DPREFIX_DIR=\"\\\"$prefix\\\"\" -Wall"
  
  if test "$DEBUG" = "1" ; then
    COMMON_CFLAGS="$COMMON_CFLAGS -DDEBUG"
  fi

  if test "$CYGWIN" = "yes" ; then
    COMMON_CFLAGS="$COMMON_CFLAGS -mwindows"
  fi

  AC_SUBST(COMMON_CFLAGS)
  AC_SUBST(GLOBAL_CFLAGS)
  
  AC_SUBST(CORE_CFLAGS)
  AC_SUBST(CORE_LDFLAGS)
  AC_SUBST(CORE_LIBS)
])


dnl *****************************************************************
dnl * Check for a few programs which we need
dnl *****************************************************************

AC_DEFUN(IMPULSE_CORE_SUMMARY,
[
  IMPULSE_ECHO([============ ROWS Configuration Summary ==============])
  IMPULSE_ECHO([])
  IMPULSE_ECHO([ Installation prefix : $prefix])
])
	

dnl End of file
