dnl
dnl Check for window system
dnl


dnl *****************************************************************
dnl * Register additional command line args
dnl *****************************************************************



AC_DEFUN(IMPULSE_GUI_INIT,
[
  AC_ARG_WITH([gui],
  [  --with-gui=[xwin/win32] Specify the graphical user interface],
  [
    case $with_gui in
     xwin)
      GUI_PREFERENCE="XWIN";;
      
     win32)
      GUI_PREFERENCE="WIN32";;
      
     *)
      GUI_PREFERENCE="NONE";;
    esac
  ],[
    GUI_PREFERENCE="NONE"
  ])
  
  GUI_XWIN_PROTECT_THREAD=0
  AC_ARG_ENABLE(xhack, [  --enable-xhack          Allow (limited) thread support for non-conformant
                          Xlibs (likely to be unstable) ],
  [
    if test "$enableval" = yes; then
      GUI_XWIN_PROTECT_THREAD=Unknown
    fi
  ])

  AC_ARG_ENABLE(forcexhack, [  --enable-forcexhack     Force the hacky X threading code (for
                          debugging purposes) ],
  [
    if test "$enableval" = yes; then
      GUI_XWIN_PROTECT_THREAD=1
    fi
  ])

  GUI_LIBRARY="none"
])


dnl *****************************************************************
dnl * Determine what gui library to use
dnl *****************************************************************



AC_DEFUN(IMPULSE_GUI_CHECK,
[
  IMPULSE_MSG_SECTION([Checking graphical user interface])

  if test "$GUI_PREFERENCE" = "NONE" -o "$GUI_PREFERENCE" = "XWIN" ; then
    if test "$GUI_LIBRARY" = "none" ; then
      IMPULSE_GUI_CHECK_XWIN
      if test "$USE_THREADS" = "1" -a "$GUI_LIBRARY" = "XWIN" ; then
        IMPULSE_GUI_CHECK_XWIN_THREAD
      fi
      IMPULSE_GUI_CHECK_XWIN_SHM
    fi
  fi

  if test "$GUI_PREFERENCE" = "NONE" -o "$GUI_PREFERENCE" = "WIN32" ; then
    if test "$GUI_LIBRARY" = "none" ; then
      IMPULSE_GUI_CHECK_WIN32
    fi
  fi
])



dnl *****************************************************************
dnl * Check to see X  actually works
dnl *****************************************************************



AC_DEFUN(IMPULSE_GUI_CHECK_XWIN,
[
  IMPULSE_PARAMS_SAVE
  XWIN_CFLAGS=""
  XWIN_LDFLAGS=""
  XWIN_LIBS=""

  AC_PATH_X
  AC_PATH_XTRA
  
  CFLAGS="$GLOBAL_CFLAGS $CORE_CFLAGS $GUI_CFLAGS $X_CFLAGS"
  LIBS="$CORE_LIBS $GUI_LIBS $X_LIBS -lX11 $X_EXTRA_LIBS -lXext"
dnl  LIBS="$CORE_LIBS $GUI_LIBS $X_LIBS $X_PRE_LIBS -lX11 $X_EXTRA_LIBS -lXext"
  LDFLAGS="$CORE_LDFLAGS $GUI_LDFLAGS"

  if test "$no_x" != "yes" ; then
    AC_CHECK_LIB(Xext, XShapeCombineMask,
    [
      LIBS="$LIBS -lXext"
      AC_CACHE_CHECK([for X header files and libraries],impulse_cv_X_ok,
      [
        AC_TRY_LINK(
	[
	  #include <X11/Xlib.h>
	  #include <X11/Xutil.h>
	  #include <X11/extensions/shape.h>
	],[
	  /* We really need to make this function a little more interesting */
	  
	  XOpenDisplay(NULL);
	],[
	  impulse_cv_X_ok="yes"
	],[
	  impulse_cv_X_ok="no"
	])
      ])
	
      if test "$impulse_cv_X_ok" = "yes" ; then
        GUI_LIBRARY="xwin"
        AC_DEFINE(HAVE_GUI_XWIN, 1)
        XWIN_CFLAGS="$X_CFLAGS"
 	XWIN_LIBS="$X_LIBS -lX11 $X_EXTRA_LIBS -lXext"
dnl 	XWIN_LIBS="$X_LIBS $X_PRE_LIBS -lX11 $X_EXTRA_LIBS -lXext"
	XWIN_LDFLAGS=""
      fi
    ])
  
    if test "$impulse_cv_X_ok" = "no" ; then
      AC_MSG_CHECKING(what's wrong)
      FAIL=""
    
      AC_TRY_COMPILE(
      [
        #include <X11/Xlib.h>
      ],,,[
        AC_MSG_RESULT(Can't find Xlib.h)
        FAIL="1"
      ])

      if test "x$FAIL" = "x" ; then
        AC_TRY_COMPILE(
        [
	  #include <X11/Xlib.h>
  	  #include <X11/Xutil.h>
        ],,,[
  	  AC_MSG_RESULT(Can't find Xutil.h)
	  FAIL="1"
	])
      fi
    
      if test "x$FAIL" = "x" ; then
        AC_TRY_COMPILE([
	  #include <X11/Xlib.h>
	  #include <X11/Xutil.h>
	  #include <X11/extensions/shape.h>
	],,,[
	  AC_MSG_RESULT(Can't find extensions/shape.h)
	  FAIL="1"
	])
      fi
      
      if test "x$FAIL" = "x" ; then
        AC_TRY_LINK([
	  #include <X11/Xlib.h>
	  #include <X11/Xutil.h>
	  #include <X11/extensions/shape.h>
	],[
	  /* We really need to make this function a little more interesting */
	  
	  XOpenDisplay(NULL);
	],[
          AC_MSG_RESULT(that's odd - it seems to work now)
	],[
	  AC_MSG_RESULT(doesn't seem to want to link)
        ])
      fi
    fi
  fi
  
  IMPULSE_PARAMS_RESTORE

  if test "$GUI_LIBRARY" = "xwin" ; then
    GUI_CFLAGS="$GUI_CFLAGS $XWIN_CFLAGS"
    GUI_LDFLAGS="$GUI_LDFLAGS $XWIN_LDFLAGS"
    GUI_LIBS="$GUI_LIBS $XWIN_LIBS"
  fi
])


dnl *****************************************************************
dnl * Check to see if Win32 is lying about anywhere
dnl *****************************************************************

AC_DEFUN(IMPULSE_GUI_CHECK_WIN32,
[
  IMPULSE_PARAMS_SAVE

  AC_CHECK_HEADER(windows.h,
  [
    GUI_LIBRARY="win32"
    AC_DEFINE(HAVE_GUI_WIN32, 1)
  ])
  
  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Check to see if we can support MIT SHM
dnl *****************************************************************


AC_DEFUN(IMPULSE_GUI_CHECK_XWIN_SHM,
[
  IMPULSE_PARAMS_SAVE

  CFLAGS="$GLOBAL_CFLAGS $CORE_CFLAGS $GUI_CFLAGS $X_CFLAGS"
  LIBS="$CORE_LIBS $GUI_LIBS $X_LIBS -lX11 $X_EXTRA_LIBS -lXext"
  LDFLAGS="$CORE_LDFLAGS $GUI_LDFLAGS"

  AC_CACHE_CHECK([for X shared memory extension support], impulse_cv_xshm, [
    AC_TRY_LINK([
      #include <X11/Xlib.h>
      #include <sys/ipc.h>
      #include <sys/shm.h>
      #include <X11/extensions/XShm.h>
    ], [
      XShmQueryExtension(NULL); /* Not very exciting... */
    ], [
      impulse_cv_xshm="yes"
    ], [
      impulse_cv_xshm="no"
    ])
  ])

  IMPULSE_PARAMS_RESTORE

  if test "$impulse_cv_xshm" = "yes" ; then
    AC_DEFINE(HAVE_XEXT_MIT_SHM, 1)
  fi
])

dnl *****************************************************************
dnl * Check to see if we have a threadsafe XWin
dnl *****************************************************************


AC_DEFUN(IMPULSE_GUI_CHECK_XWIN_THREAD,
[
  IMPULSE_PARAMS_SAVE

  CFLAGS="$GLOBAL_CFLAGS $CORE_CFLAGS $GUI_CFLAGS"
  LDFLAGS="$CORE_LDFLAGS $GUI_LDFLAGS"
  LIBS="$CORE_LIBS $GUI_LIBS"

  AC_CACHE_CHECK([for threadsafe Xlib], impulse_cv_xlibthreads,
  [
    AC_TRY_RUN([
      #include <stdio.h>
      
      #include <X11/Xlib.h>
      #include <X11/Xutil.h>
      
      int main(void)
      {
        Status st;
	
	st = XInitThreads();
	if (st != 0)
  	  return 0;
	return 1;
      }
    ],[
      impulse_cv_xlibthreads="yes"
    ],[
      impulse_cv_xlibthreads="no"
    ],[
      impulse_cv_xlibthreads="no (guessing for x-compile)"
    ])
  ])

  if test "$impulse_cv_xlibthreads" = "no" -a "$THREAD_PROTECT_X" != 0 ; then
    AC_DEFINE(HAVE_GUI_XWIN_PROTECT, 1)
  fi

  IMPULSE_PARAMS_RESTORE
])


dnl *****************************************************************
dnl * Substitute all the variables we need
dnl *****************************************************************


AC_DEFUN(IMPULSE_GUI_FINISH,
[
  AC_SUBST(GUI_CFLAGS)
  AC_SUBST(GUI_LIBS)
  AC_SUBST(GUI_LDFLAGS)
])


dnl *****************************************************************
dnl * Print out a configuration summary
dnl *****************************************************************


AC_DEFUN(IMPULSE_GUI_SUMMARY,
[
  IMPULSE_ECHO(["Graphical user interface: $GUI_LIBRARY"])
])
